from django import forms
from django.forms.util import ErrorList
from django.utils.safestring import mark_safe

from pki.models import *
from openssl import md5_constructor
import re

##------------------------------------------------------------------##
## Form validation
##------------------------------------------------------------------##

class CertificateAuthorityForm(forms.ModelForm):
    """Validation class for CertificateAuthority form"""
    
    passphrase        = forms.CharField(widget=forms.PasswordInput)
    parent_passphrase = forms.CharField(widget=forms.PasswordInput, required=False)
    
    class Meta:
        model = CertificateAuthority
    
    def clean(self):
        """Verify crucial fields"""
        
        cleaned_data = self.cleaned_data
        
        action = cleaned_data.get('action')
        parent = cleaned_data.get('parent')
        pf = cleaned_data.get('passphrase')
        enc_p_pf = None

        if action in ('create', 'renew'):
            ## Check if name contains invalid chars
            name = cleaned_data.get('name')
            
            if name != None and re.search('[^a-zA-Z0-9-_\.]', name):
                self._errors['name'] = ErrorList(['Name may only contain characters in range a-Z0-9_-.'])
            
            ## Verify passphrase length
            if action == 'create' and pf and len(pf) < 8:
                self._errors['passphrase'] = ErrorList(['Passphrase has to be at least 8 characters long'])
            
            ## Take care that parent is active when action is revoke
            if action == 'renew':
                ca = CertificateAuthority.objects.get(name='%s' % name)
                
                if ca.parent is not None and ca.parent.active is not True:
                    self._errors['action'] = ErrorList(['Cannot renew CA certificate when parent "%s" isn\'t active!' % ca.parent.name])
                    return cleaned_data
                
                ## Self-signed renew. compare passphrase
                if ca.parent is None:
                    if not pf or ca.passphrase != md5_constructor(pf).hexdigest():
                        self._errors['passphrase'] = ErrorList(['Passphrase is wrong. Enter correct passphrase for CA "%s"' % name])
            
            if parent:                
                ca = CertificateAuthority.objects.get(name='%s' % parent)
                p_pf = cleaned_data.get('parent_passphrase')
                if p_pf: enc_p_pf = md5_constructor(p_pf).hexdigest()
                
                ## Check if parent allows sub CA
                if not ca.subcas_allowed:
                    self._errors['parent'] = ErrorList(['Parent CA %s doesn\'t allow a sub CA. Only non CA certificates can be created' % ca.name])
                    
                ## Check parent passphrase if not RootCA
                if ca.passphrase != enc_p_pf:
                    self._errors['parent_passphrase'] = ErrorList(['Passphrase is wrong. Enter correct passphrase for CA "%s"' % parent])
        
        elif action == 'revoke':
            
            if parent:
                ca = CertificateAuthority.objects.get(name='%s' % parent)
                enc_p_pf = md5_constructor(cleaned_data.get('parent_passphrase')).hexdigest()
                
                ## Check parent passphrase
                if ca.passphrase != enc_p_pf:
                    self._errors['parent_passphrase'] = ErrorList(['Passphrase is wrong. Enter correct passphrase for CA %s' % parent])
            else:
                self._errors['action'] = ErrorList(['You cannot revoke a self-signed root certificate as this would break the whole chain!'])
        
        return cleaned_data

class CertificateForm(forms.ModelForm):
    """Validation class for Certificate form"""
    
    passphrase        = forms.CharField(widget=forms.PasswordInput, required=False)
    parent_passphrase = forms.CharField(widget=forms.PasswordInput, required=False)
    pkcs12_passphrase = forms.CharField(widget=forms.PasswordInput, required=False)
    
    class Meta:
        model = Certificate
    
    def clean(self):
        """Verify crucial fields"""
        
        cleaned_data = self.cleaned_data
        
        action = cleaned_data.get('action')
        parent = cleaned_data.get('parent')
        pf = cleaned_data.get('passphrase')
        p_pf = cleaned_data.get('parent_passphrase')
        subjaltname = cleaned_data.get('subjaltname')
        pkcs12_passphrase = cleaned_data.get('pkcs12_passphrase')
        pkcs12_encoded = cleaned_data.get('pkcs12_encoded')
        
        enc_p_pf = None
        
        if action in ('create', 'renew'):
            ## Check if name contains invalid chars
            name = cleaned_data.get('name')
            
            if name != None and re.search('[^a-zA-Z0-9-_\.]', name):
                self._errors['name'] = ErrorList(['Name may only contain characters in range a-Z0-9'])
            
            ## Verify passphrase length
            if action == 'create' and pf and len(pf) < 8:
                self._errors['passphrase'] = ErrorList(['Passphrase has to be at least 8 characters long'])
            
            ## Verify that pkcs12 passphrase isn't empty when encoding is requested
            if pkcs12_encoded and len(pkcs12_passphrase) < 8:
                self._errors['pkcs12_passphrase'] = ErrorList(['PKCS12 passphrase has to be at least 8 characters long'])
            
            ## Take care that parent is active when action is revoke
            if action == 'renew':
                cert = Certificate.objects.get(name='%s' % name)
                
                if cert.parent is not None and cert.parent.active is not True:
                    self._errors['action'] = ErrorList(['Cannot renew certificate when parent CA "%s" isn\'t active!' % cert.parent.name])
                    return cleaned_data
            
            if parent:
                ca = CertificateAuthority.objects.get(name='%s' % parent)
                if p_pf: enc_p_pf = md5_constructor(p_pf).hexdigest()
                
                ## Check parent passphrase
                if ca.passphrase != enc_p_pf:
                    self._errors['parent_passphrase'] = ErrorList(['Passphrase is wrong. Enter correct passphrase for CA %s' % parent])
            else:
                self._errors['parent'] = ErrorList(['You cannot renew a certificate while the parent is not active. Renew requires the intial parent to be active'])
            
            ## Verify subjAltName
            if subjaltname and len(subjaltname) > 0:
                allowed = { 'email': '^copy|[\w\-\.]+\@[\w\-\.]+\.\w{2,4}$',
                            'IP'   : '^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$',
                            'DNS'  : '^[a-zA-Z0-9\-\.\*]+$',
                          }
                items = subjaltname.split(',')
                
                for i in items:
                    if not re.match( '^\s*(email|IP|DNS)\s*:\s*.+$', i):
                        self._errors['subjaltname'] = ErrorList(['Item "%s" doesn\'t match specification' % i])
                    else:
                        kv  = i.split(':')
                        key = kv[0].lstrip().rstrip()
                        val = kv[1].lstrip().rstrip()
                        
                        if key in allowed:
                            if not re.match( allowed[key], val ):
                                self._errors['subjaltname'] = ErrorList(['Invalid subjAltName value supplied: \"%s\"' % i])
                        else:
                            self._errors['subjaltname'] = ErrorList(['Invalid subjAltName key supplied: "%s" (supported are %s)' % (key, ', '.join(allowed.keys()))])
        elif action == 'revoke':
            if parent:
                ca = CertificateAuthority.objects.get(name='%s' % parent)
                if p_pf: enc_p_pf = md5_constructor(p_pf).hexdigest()
                
                ## Check parent passphrase
                if ca.passphrase != enc_p_pf:
                    self._errors['parent_passphrase'] = ErrorList(['Passphrase is wrong. Enter correct passphrase for CA %s' % parent])
        elif action == 'update':
            ## Verify that pkcs12 passphrase isn't empty when encoding is requested
            if pkcs12_encoded and len(pkcs12_passphrase) < 8:
                self._errors['pkcs12_passphrase'] = ErrorList(['PKCS12 passphrase has to be at least 8 characters long'])
        
        return cleaned_data

##------------------------------------------------------------------##
## Dynamic forms
##------------------------------------------------------------------##

class CaPassphraseForm(forms.Form):
    passphrase = forms.CharField(max_length=100, widget=forms.PasswordInput)
    ca_id      = forms.CharField(widget=forms.HiddenInput)
    
    def clean(self):
        """Verify crucial fields"""
        
        cleaned_data = self.cleaned_data
        passphrase   = cleaned_data.get('passphrase')
        
        if passphrase: 
            e_passphrase = md5_constructor(cleaned_data.get('passphrase')).hexdigest()
            ca_id        = cleaned_data.get('ca_id')
            ca           = CertificateAuthority.objects.get(pk=ca_id)
            
            if ca.passphrase != e_passphrase:
                self._errors["passphrase"] = ErrorList(['Passphrase is wrong. Enter correct passphrase for CA %s' % ca.name])
        else:
            self._errors["passphrase"] = ErrorList(['Passphrase is missing!'])
        
        return cleaned_data

##------------------------------------------------------------------##
## Readonly setup
##------------------------------------------------------------------##

class ReadOnlyWidget(forms.Widget):
    def __init__(self, original_value, display_value):
        self.original_value = original_value
        self.display_value = display_value
        
        super(ReadOnlyWidget, self).__init__()
    
    def render(self, name, value, attrs=None):
        if self.display_value is not None:
            return mark_safe(unicode(self.display_value))
        return mark_safe(unicode(self.original_value))

    def value_from_datadict(self, data, files, name):
        return self.original_value

class ReadOnlyAdminFields(object):
    def get_form(self, request, obj=None):
        form = super(ReadOnlyAdminFields, self).get_form(request, obj)
        
        if hasattr(self, 'readonly'):
            for field_name in self.readonly:
                if field_name in form.base_fields:
                    
                    if hasattr(obj, 'get_%s_display' % field_name):
                        display_value = getattr(obj, 'get_%s_display' % field_name)()
                    else:
                        display_value = None
                        
                    form.base_fields[field_name].widget = ReadOnlyWidget(getattr(obj, field_name, ''), display_value)
                    form.base_fields[field_name].required = False
        
        return form



from django.conf.urls.defaults import *
from pki.views import *

urlpatterns = patterns('',
    (r'admin/pki/(?P<model>certificate|certificateauthority)/(?P<id>[0-9]+)/delete/', admin_delete),
    (r'^pki/download/(?P<type>ca|cert)/(?P<id>\d+)/$', pki_download),
    (r'^pki/chain/(?P<type>ca|cert)/(?P<id>\d+)/$', pki_chain),
    (r'^pki/tree/(?P<id>\d+)/$', pki_tree),
    (r'^pki/email/(?P<type>ca|cert)/(?P<id>\d+)/$', pki_email),
)



import os

from django.contrib import admin
from django.utils.safestring import mark_safe

from pki.models import CertificateAuthority, Certificate
from pki.forms import ReadOnlyAdminFields, CertificateAuthorityForm, CertificateForm
from pki.settings import PKI_DIR, PKI_LOG, PKI_LOGLEVEL, JQUERY_URL

##------------------------------------------------------------------##
## Create PKI_DIR if it's missing
##------------------------------------------------------------------##

if not os.path.exists( PKI_DIR ):
    try:
        os.mkdir( PKI_DIR, 0750 )
    except OSError, e:
        print "Failed to create PKI_DIR %s: %s" % (PKI_DIR, e)

##------------------------------------------------------------------##
## Initialize logging
##------------------------------------------------------------------##

import logging

LOG_LEVELS = { 'debug'    : logging.DEBUG,
               'info'     : logging.INFO,
               'warning'  : logging.WARNING,
               'error'    : logging.ERROR,
               'critical' : logging.CRITICAL
             }

logger = logging.getLogger("pki")

l_hdlr = logging.FileHandler(PKI_LOG)
l_hdlr.setFormatter(logging.Formatter("%(asctime)s %(levelname)s - %(module)s.%(funcName)s > %(message)s"))

if LOG_LEVELS[PKI_LOGLEVEL]:
    logger.setLevel(LOG_LEVELS[PKI_LOGLEVEL])

logger.addHandler(l_hdlr)

##---------------------------------##
## Interface setup
##---------------------------------##

class Certificate_Authority_Admin(ReadOnlyAdminFields, admin.ModelAdmin):
    """CertificateAuthority admin definition"""
    form               = CertificateAuthorityForm
    list_display       = ( 'id', 'common_name', 'Serial_align_right', 'active_center', 'Chain_link', 'Tree_link', 'Parent_link',
                           'Expiry_date', 'Description', 'type', 'revoked', 'Download_link', 'Email_link', )
    list_display_links = ( 'common_name', )
    save_on_top        = True
    actions            = []
    list_filter        = ( 'parent', 'active', )
    radio_fields       = { "action": admin.VERTICAL }
    search_fields      = [ 'name', 'common_name', 'description' ]
    date_hierarchy     = 'created'
    readonly           = ( 'expiry_date', 'key', 'cert', 'serial', 'ca_chain', )
    exclude            = ( 'pf_encrypted', 'pem_encoded', )
    fieldsets          = ( ( 'Define action',    { 'fields': ( 'action', ) }, ),
                           ( 'Documentation',    { 'fields': ( 'description', ) } ), 
                           ( 'Certificate',      { 'fields': ( 'common_name', 'name', 'country', 'state', 'locality', 'organization', 'OU',
                                                               'email', 'key_length', 'valid_days', 'passphrase', 'serial', 'expiry_date',
                                                             )
                                                 }
                           ),
                           ( 'Encoding options', { 'fields': ( 'der_encoded', ), } ),
                           ( 'CA setup',         { 'fields': ( 'subcas_allowed', 'ca_chain', 'parent', 'type', 'parent_passphrase', 'policy', ), } ),
                         )
    
    class Media:
        js = ( JQUERY_URL, 'pki/ca_admin.js', )
    
    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        """Filter foreign key parent field.
        
        Skip CAs that dont have subcas_allowed set or are not active
        """
        
        if db_field.name == "parent":
            kwargs["queryset"] = CertificateAuthority.objects.filter(subcas_allowed=True, active=True)
            return db_field.formfield(**kwargs)
        
        return super(Certificate_Authority_Admin, self).formfield_for_foreignkey(db_field, request, **kwargs)

admin.site.register(CertificateAuthority, Certificate_Authority_Admin)

class Certificate_Admin(ReadOnlyAdminFields, admin.ModelAdmin):
    """CertificateAuthority admin definition"""
    form               = CertificateForm
    list_display       = ( 'id', 'common_name', 'Serial_align_right', 'active_center', 'Chain_link', 'Parent_link',
                           'Expiry_date', 'Description', 'created', 'revoked', 'Download_link', 'Email_link' )
    list_display_links = ( 'common_name', )
    save_on_top        = True
    actions            = []
    radio_fields       = { "action": admin.VERTICAL }
    list_filter        = ( 'parent', 'active', )
    search_fields      = [ 'name', 'description' ]
    date_hierarchy     = 'created'
    readonly           = ( 'created', 'expiry_date', 'key', 'cert', 'serial', 'ca_chain', )
    exclude            = ( 'pf_encrypted', )
    fieldsets          = ( ( 'Define action',   { 'fields': ( 'action', ) } ),
                           ( 'Documentation',   { 'fields': ( 'description', ) } ), 
                           ( 'Certificate',     { 'fields': ( 'common_name', 'name', 'country', 'state', 'locality', 'organization', 'OU',
                                                              'email', 'subjaltname', 'key_length', 'cert_extension', 'valid_days', 'passphrase', 'serial', 'expiry_date',
                                                            ),
                                                }
                           ),
                           ( 'Encoding options', { 'fields': ( 'der_encoded', 'pkcs12_encoded', 'pkcs12_passphrase', ), } ),
                           ( 'CA setup',         { 'fields': ( 'ca_chain', 'parent', 'parent_passphrase', ), } ),
                         )
    
    class Media:
        js = ( JQUERY_URL, 'pki/cert_admin.js', )
    
    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        """Filter foreign key parent field.
        
        Skip CAs that dont have subcas_allowed set or are not active
        """
        
        if db_field.name == "parent":
            kwargs["queryset"] = CertificateAuthority.objects.filter(subcas_allowed=False, active=True)
            return db_field.formfield(**kwargs)
        
        return super(Certificate_Authority_Admin, self).formfield_for_foreignkey(db_field, request, **kwargs)

admin.site.register(Certificate, Certificate_Admin)
admin.site.disable_action('delete_selected')




# Generated by the windmill services transformer
from windmill.authoring import WindmillTestClient

def test_ViewsWithoutLogin():
    client = WindmillTestClient(__name__)

    client.open(url=u'http://127.0.0.1:8000/admin/logout/')
    client.waits.forPageLoad(timeout=u'8000')
    client.open(url=u'http://127.0.0.1:8000/pki/download/ca/1/')
    client.waits.forPageLoad(timeout=u'8000')
    client.asserts.assertNode(xpath=u"//form[@id='login-form']/div[4]")
    client.open(url=u'http://127.0.0.1:8000/pki/download/cert/1/')
    client.waits.forPageLoad(timeout=u'8000')
    client.asserts.assertNode(xpath=u"//form[@id='login-form']/div[4]")



##############################################################
## PKI settings
##############################################################

import os
from django.conf import settings

PKI_APP_DIR = os.path.abspath(os.path.dirname(__file__))

# base directory for pki storage (should be writable), defaults to PKI_APP_DIR/PKI
PKI_DIR = getattr(settings, 'PKI_DIR', os.path.join(PKI_APP_DIR, 'PKI'))

# path to openssl executable
PKI_OPENSSL_BIN = getattr(settings, 'PKI_OPENSSL_BIN', '/usr/bin/openssl')

# path to generated openssl.conf
PKI_OPENSSL_CONF = getattr(settings, 'PKI_OPENSSL_CONF', os.path.join(PKI_DIR, 'openssl.conf'))

# template name for openssl.conf
PKI_OPENSSL_TEMPLATE = getattr(settings, 'PKI_OPENSSL_TEMPLATE', 'pki/openssl.conf.in')

# jquery url (defaults to pki/jquery-1.3.2.min.js)
JQUERY_URL = getattr(settings, 'JQUERY_URL', 'pki/jquery-1.4.2.min.js')

# logging (TODO: syslog, handlers and formatters)
PKI_LOG = getattr(settings, 'PKI_LOG', os.path.join(PKI_DIR, 'pki.log'))
PKI_LOGLEVEL = getattr(settings, 'PKI_LOGLEVEL', 'debug')

# get other settings directly from settings.py:
ADMIN_MEDIA_PREFIX = getattr(settings, 'ADMIN_MEDIA_PREFIX')

# media url
MEDIA_URL = getattr(settings, 'MEDIA_URL')

# base url: without trailing slash. Leave empty if mounted on /
PKI_BASE_URL = getattr(settings, 'PKI_BASE_URL', '')

# self_signed_serial; The serial a self signed CA starts with. Set to 0 or 0x0 for a random number
PKI_SELF_SIGNED_SERIAL = getattr(settings, 'PKI_SELF_SIGNED_SERIAL', 0x0)

# default_country: The default country selected (2-letter country code)
PKI_DEFAULT_COUNTRY = getattr(settings, 'PKI_DEFAULT_COUNTRY', 'DE')

# enable graphviz_support: When True django-pki will render Graphviz PNG's to show relations
PKI_ENABLE_GRAPHVIZ = getattr(settings, 'PKI_ENABLE_GRAPHVIZ', False)

# graphviz direction: From left to right (LR) or top down (TD)
PKI_GRAPHVIZ_DIRECTION = getattr(settings, 'PKI_GRAPHVIZ_DIRECTION', 'LR')

# enable email delivery: Certificates with defined email address can be sent via email
PKI_ENABLE_EMAIL = getattr(settings, 'PKI_ENABLE_EMAIL', False)




# Generated by the windmill services transformer
from windmill.authoring import WindmillTestClient

def test_create_certificate():
    client = WindmillTestClient(__name__)

    client.open(url=u'http://127.0.0.1:8000/admin/logout/')
    client.waits.forPageLoad(timeout=u'8000')
    client.open(url=u'http://127.0.0.1:8000/admin/')
    client.waits.forPageLoad(timeout=u'8000')
    client.waits.forElement(timeout=u'8000', id=u'id_username')
    client.click(id=u'id_username')
    client.type(text=u'admin', id=u'id_username')
    client.type(text=u'admin', id=u'id_password')
    client.click(value=u'Log in')
    client.waits.forPageLoad(timeout=u'20000')
    client.waits.forElement(link=u'Certificates', timeout=u'8000')
    client.click(link=u'Certificates')
    client.waits.forPageLoad(timeout=u'20000')
    client.waits.forElement(link=u'              Add certificate             ', timeout=u'8000')
    client.click(link=u'              Add certificate             ')
    client.waits.forPageLoad(timeout=u'20000')
    client.waits.forElement(timeout=u'8000', id=u'id_description')
    client.click(id=u'id_description')
    client.type(text=u'srv1', id=u'id_description')
    client.type(text=u'srv1.mycompany.com', id=u'id_common_name')
    client.click(id=u'id_state')
    client.type(text=u'Bavaria', id=u'id_state')
    client.type(text=u'Munich', id=u'id_locality')
    client.type(text=u'My Company', id=u'id_organization')
    client.type(text=u'Datacenter', id=u'id_OU')
    client.click(id=u'id_email')
    client.type(text=u'a@b.com', id=u'id_email')
    client.click(id=u'id_subjaltname')
    client.type(text=u'DNS:*.mycompany.com', id=u'id_subjaltname')
    client.click(id=u'id_key_length')
    client.select(option=u'', id=u'id_key_length')
    client.click(value=u'1024')
    client.click(id=u'id_cert_extension')
    client.select(option=u'', id=u'id_cert_extension')
    client.click(value=u'v3_server_cert')
    client.click(id=u'id_valid_days')
    client.type(text=u'365', id=u'id_valid_days')
    client.click(id=u'id_parent')
    client.select(option=u'', id=u'id_parent')
    client.click(value=u'2')
    client.click(id=u'id_parent_passphrase')
    client.type(text=u'1234567890', id=u'id_parent_passphrase')
    client.click(xpath=u"//form[@id='certificate_form']/div[2]/div[2]/input[1]")
    client.waits.forPageLoad(timeout=u'20000')
    client.asserts.assertTextIn(xpath=u"//div[@id='container']/ul/li", validator=u'The certificate "srv1.mycompany.com" was added successfully.')



import os
import logging

from django.contrib.auth.decorators import login_required
from django.shortcuts import render_to_response, get_object_or_404
from django.http import HttpResponse, Http404, HttpResponseRedirect, HttpResponseBadRequest
from django.utils.safestring import mark_safe
from django.template import RequestContext

from pki.settings import PKI_LOG, MEDIA_URL, PKI_ENABLE_GRAPHVIZ, PKI_ENABLE_EMAIL
from pki.models import CertificateAuthority, Certificate
from pki.forms import CaPassphraseForm
from pki.graphviz import ObjectChain, ObjectTree
from pki.email import SendCertificateData
from pki.helper import files_for_object, chain_recursion, build_delete_item, generate_temp_file, build_zip_for_object

logger = logging.getLogger("pki")

##------------------------------------------------------------------##
## Download views
##------------------------------------------------------------------##

@login_required
def pki_download(request, type, id):
    """Download PKI data.
    
    Type (ca/cert) and ID are used to determine the object to download.
    """
    
    if type == "ca":
        c = get_object_or_404(CertificateAuthority, pk=id)
    elif type == "cert":
        c = get_object_or_404(Certificate, pk=id)
    else:
        logger.error( "Unsupported type %s requested!" % type )
        return HttpResponseBadRequest()
    
    if not c.active:
        raise Http404
    
    zip = build_zip_for_object(c, request)
    
    ## open and read the file if it exists
    if os.path.exists(zip):
        f = open(zip)
        x = f.readlines()
        f.close()
        
        ## return the HTTP response
        response = HttpResponse(x, mimetype='application/force-download')
        response['Content-Disposition'] = 'attachment; filename="PKI_DATA_%s.zip"' % c.name
        
        return response
    else:
        logger.error( "File not found: %s" % zip )
        raise Http404

##------------------------------------------------------------------##
## Graphviz views
##------------------------------------------------------------------##

@login_required
def pki_chain(request, type, id):
    """Display the CA chain as PNG.
    
    Requires PKI_ENABLE_GRAPHVIZ set to true. Type (ca/cert) and ID are used to determine the object.
    Create object chain PNG using graphviz and return it to the user.
    """
    
    if PKI_ENABLE_GRAPHVIZ is not True:
        raise Exception( "Chain view is inoperable unless PKI_ENABLE_GRAPHVIZ is enabled" )
    
    if type == "ca":
        obj = get_object_or_404(CertificateAuthority, pk=id)
    elif type == "cert":
        obj = get_object_or_404(Certificate, pk=id)
    
    png = generate_temp_file()

    ObjectChain(obj, png)
    
    try:
        if os.path.exists(png):
            f = open(png)
            x = f.read()
            f.close()
            
            os.remove(png)
    except OSError,e:
        logger.error( "Failed to load depency tree: %s" % e)
        raise Exception( e )
    
    response = HttpResponse(x, mimetype='image/png')
    return response

@login_required
def pki_tree(request, id):
    """Display the CA tree as PNG.
    
    Requires PKI_ENABLE_GRAPHVIZ set to true. Only works for Certificate Authorities.
    All object related to the CA obj are fetched and displayed in a Graphviz tree.
    """
    
    if PKI_ENABLE_GRAPHVIZ is not True:
        raise Exception( "Tree view is inoperable unless PKI_ENABLE_GRAPHVIZ is enabled!" )
    
    obj = get_object_or_404(CertificateAuthority, pk=id)
    png = generate_temp_file()
    
    ObjectTree(obj, png)
    
    try:
        if os.path.exists(png):
            f = open(png)
            x = f.read()
            f.close()
            
            os.remove(png)
    except OSError,e:
        logger.error( "Failed to load depency tree: %s" % e)
        raise Exception( e )
    
    response = HttpResponse(x, mimetype='image/png')
    return response

##------------------------------------------------------------------##
## Email views
##------------------------------------------------------------------##

@login_required
def pki_email(request, type, id):
    """Send email with certificate data attached.
    
    Requires PKI_ENABLE_EMAIL set to true. Type (ca/cert) and ID are used to determine the object.
    Build ZIP, send email and return to changelist.
    """
    
    if PKI_ENABLE_EMAIL is not True:
        raise Exception( "Email sending is inoperable unless PKI_ENABLE_EMAIL is enabled!" )
    
    if type == "ca":
        obj  = get_object_or_404(CertificateAuthority, pk=id)
    elif type == "cert":
        obj = get_object_or_404(Certificate, pk=id)
    
    back = request.META.get('HTTP_REFERER', None) or '/'
    
    if obj.email and obj.active:
        SendCertificateData(obj, request)
    else:
        raise Http404
    
    request.user.message_set.create(message='Email to "%s" was sent successfully.' % obj.email)
    return HttpResponseRedirect(back)

##------------------------------------------------------------------##
## Admin views
##------------------------------------------------------------------##

@login_required
def admin_delete(request, model, id):
    """Overwite the default admin delete view"""
    
    deleted_objects    = []
    parent_object_name = CertificateAuthority._meta.verbose_name
    title              = 'Are you sure?'
    
    if model == 'certificateauthority':
        ## Get the list of objects to delete as list of lists
        item = get_object_or_404(CertificateAuthority, pk=id)
        chain_recursion(item.id, deleted_objects, id_dict={ 'cert': [], 'ca': [], })
        
        ## Fill the required data for delete_confirmation.html template
        opts               = CertificateAuthority._meta
        object             = item.name
        initial_ca_id      = False
        
        ## Set the CA to verify the passphrase against
        if item.parent_id:
            initial_ca_id = item.parent_id
            auth_object   = CertificateAuthority.objects.get(pk=item.parent_id).name
        else:
            initial_ca_id = item.pk
            auth_object   = item.name
    elif model == 'certificate':
        ## Fetch the certificate data
        try:
            item = Certificate.objects.select_related().get(pk=id)
        except:
            raise Http404
        
        div_content = build_delete_item(item)
        deleted_objects.append( mark_safe('Certificate: <a href="../../../certificate/%d/">%s</a> <img src="%spki/img/plus.png" class="switch" /><div class="details">%s</div>' % (item.pk, item.name, MEDIA_URL, div_content)) )
        
        ## Fill the required data for delete_confirmation.html template
        opts               = Certificate._meta
        object             = item.name
        initial_ca_id      = item.parent_id
        
        ## Set the CA to verify the passphrase against
        auth_object = item.parent.name
    
    if request.method == 'POST':
        form = CaPassphraseForm(request.POST)
        
        if form.is_valid():
            item.delete(request.POST['passphrase'])
            request.user.message_set.create(message='The %s "%s" was deleted successfully.' % (opts.verbose_name, object))
            return HttpResponseRedirect("../../")
    else:
        form = CaPassphraseForm()
        form.fields['ca_id'].initial = initial_ca_id
    
    return render_to_response('admin/pki/delete_confirmation.html', { 'deleted_objects': deleted_objects, 'object_name': opts.verbose_name,
                                                                      'app_label': opts.app_label, 'opts': opts, 'object': object, 'form': form,
                                                                      'auth_object': auth_object, 'parent_object_name': parent_object_name,
                                                                      'title': title,
                                                                    }, RequestContext(request))

@login_required
def show_exception(request):
    """Render error page and fill it with the PKI_LOG content"""
    
    f = open(PKI_LOG, 'r')
    log = f.readlines()
    f.close()
    
    return render_to_response('500.html', {'log': log})



import os
import tempfile
import string
import random
import zipfile
import logging

from django.utils.safestring import mark_safe

from pki.settings import PKI_DIR, MEDIA_URL
import pki.models

logger = logging.getLogger("pki")

def files_for_object(obj):
    """Return files associated to object.
    
    Return dict containing all files associated to object. Dict contains
    chain, crl, pem, csr, der, pkcs12 and key
    """
    
    if isinstance( obj, pki.models.CertificateAuthority):
        chain   = c_name = obj.name
        ca_dir  = os.path.join(PKI_DIR, obj.name)
        key_loc = os.path.join(ca_dir, 'private')
    elif isinstance( obj, pki.models.Certificate):
        chain   = obj.parent.name
        c_name  = obj.name
        ca_dir  = os.path.join(PKI_DIR, obj.parent.name)
        key_loc = os.path.join(ca_dir, 'certs')
    else:
        raise Exception( "Given object type is unknown!" )
    
    files = { 'chain' : { 'path': os.path.join(ca_dir, '%s-chain.cert.pem' % chain),
                          'name': '%s-chain.cert.pem' % chain,
                        },
              'crl'   : { 'path': os.path.join(ca_dir, 'crl', '%s.crl.pem' % c_name),
                          'name': '%s.crl.pem' % c_name,
                        },
              'pem'   : { 'path': os.path.join(ca_dir, 'certs', '%s.cert.pem' % c_name),
                          'name': '%s.cert.pem' % c_name,
                        },
              'csr'   : { 'path': os.path.join(ca_dir, 'certs', '%s.csr.pem' % c_name),
                          'name': '%s.csr.pem' % c_name,
                        },
              'der'   : { 'path': os.path.join(ca_dir, 'certs', '%s.cert.der' % c_name),
                          'name': '%s.cert.der' % c_name,
                        },
              'pkcs12': { 'path': os.path.join(ca_dir, 'certs', '%s.cert.p12' % c_name),
                          'name': '%s.cert.p12' % c_name,
                        },
              'key'   : { 'path': os.path.join(ca_dir, key_loc, '%s.key.pem' % c_name),
                          'name': '%s.key.pem' % c_name,
                        },
            }
    
    return files

def subject_for_object(obj):
    """Return a subject string.
    
    A OpenSSL compatible subject string is returned.
    """
    
    subj = '/CN=%s/C=%s/ST=%s/localityName=%s/O=%s' % ( obj.common_name,
                                                        obj.country,
                                                        obj.state,
                                                        obj.locality,
                                                        obj.organization,
                                                      )
    
    if obj.OU:
        subj += '/organizationalUnitName=%s' % obj.OU
    
    if obj.email:
        subj += '/emailAddress=%s' % obj.email
    
    return subj

def chain_recursion(r_id, store, id_dict):
    """Helper function for recusion"""
    
    i = pki.models.CertificateAuthority.objects.get(pk=r_id)
    
    div_content = build_delete_item(i)
    store.append( mark_safe('Certificate Authority: <a href="../../%d/">%s</a> <img src="%spki/img/plus.png" class="switch" /><div class="details">%s</div>' % (i.pk, i.name, MEDIA_URL, div_content)) )
    
    id_dict['ca'].append(i.pk)
    
    ## Search for child certificates
    child_certs = pki.models.Certificate.objects.filter(parent=r_id)
    if child_certs:
        helper = []
        for cert in child_certs:
            div_content = build_delete_item(cert)
            helper.append( mark_safe('Certificate: <a href="../../../certificate/%d/">%s</a> <img src="%spki/img/plus.png" class="switch" /><div class="details">%s</div>' % (cert.pk, cert.name, MEDIA_URL, div_content)) )
            id_dict['cert'].append(cert.pk)
        store.append(helper)
    
    ## Search for related CA's
    child_cas = pki.models.CertificateAuthority.objects.filter(parent=r_id)
    if child_cas:
        helper = []
        for ca in child_cas:
            chain_recursion(ca.pk, helper, id_dict)
        store.append(helper)

def build_delete_item(obj):
    """Build div tag for delete details"""
    
    parent = 'None'
    if obj.parent is not None:
        parent = obj.parent.name
    
    return "<ul><li>Serial: %s</li><li>Subject: %s</li><li>Parent: %s</li><li>Description: %s</li><li>Created: %s</li><li>Expiry date: %s</li></ul>" % ( obj.serial, subject_for_object(obj), parent, obj.description, obj.created, obj.expiry_date)

def generate_temp_file():
    """Generate a filename in the systems temp directory"""
    
    f = os.path.join(tempfile.gettempdir(), "".join(random.sample(string.letters+string.digits, 25)))
    
    if os.path.exists(f):
        raise Exception( "The generated temp file %s already exists!" % f )
    
    return f

def build_zip_for_object(obj, request):
    """Build zip with filed ob object.
    
    request is required to check permissions. Zip file path is returned.
    """
    
    try:
        ## Create the ZIP archive
        base_folder = 'PKI_DATA_%s' % obj.name
        files       = files_for_object(obj)
        zip_f       = generate_temp_file()
        
        c_zip = zipfile.ZipFile(zip_f, 'w')
        
        ## Private key is only included if user has permission
        if not request.user.has_perm('pki.can_download_%s' % type):
            logger.error( "Permission denied: Private key is excluded" )
        else:
            logger.debug( "Access granted. User is allowed to download private key" )
            c_zip.write( files['key']['path'], files['key']['name'] )
        
        c_zip.write( files['pem']['path'], files['pem']['name'] )
        c_zip.write( files['chain']['path'], files['chain']['name'])
        
        try:
            if obj.pkcs12_encoded:
                c_zip.write( files['pkcs12']['path'], files['pkcs12']['name'] )
        except AttributeError:
            pass
        
        if obj.der_encoded:
            c_zip.write( files['der']['path'], files['der']['name'] )
        
        c_zip.close()
    except Exception, e:
        logger.error( "Exception during zip file creation: %s" % e )
        raise Exception( e )
    
    return zip_f




import unittest
import datetime
import os
import sys

from django.core.mail import get_connection
from django.test.client import Client
from django.test import TestCase

from windmill.authoring import djangotest 

##-----------------------------------------##
## <MonkeyPatching: ugly but necessary>
##-----------------------------------------##
from pki.settings import PKI_DIR, PKI_ENABLE_EMAIL
PKI_DIR = os.path.join( PKI_DIR, '../TEST_PKI')

from pki.models  import CertificateAuthority
from pki import openssl
from pki.helper import *

openssl.PKI_DIR = PKI_DIR
openssl.PKI_OPENSSL_CONF = os.path.join(PKI_DIR, 'openssl.conf')
##-----------------------------------------##
## </MonkeyPatching>
##-----------------------------------------##

## Logging to STDOUT on level ERROR
import logging

logger = logging.getLogger("pki")

l_hdlr = logging.FileHandler(os.path.join(PKI_DIR, 'pki.log'))
l_hdlr.setFormatter(logging.Formatter("%(asctime)s %(levelname)s - %(module)s.%(funcName)s > %(message)s"))
logger.addHandler(l_hdlr)
logger.setLevel(logging.DEBUG)

##-----------------------------------------##
## Create admin user
##-----------------------------------------##

class CertificateAuthorityTestCase(unittest.TestCase):
    '''Testcase for a self-signed RootCA. Any affected function and the complete process (save+remove) are tested''' 
    
    def setUp(self):
        '''Create a self-signed RootCA'''
        
        ## Root CA object
        self.rca = CertificateAuthority( common_name='Root CA', name='Root_CA', description="unit test", country='DE', state='Bavaria', \
                                         locality='Munich', organization='Bozo Clown Inc.', OU='IT', email='a@b.com', valid_days=1000, \
                                         key_length=1024, expiry_date='', created='', revoked=None, active=None, serial=None, ca_chain=None, \
                                         pem_encoded=True, der_encoded=False, parent=None, passphrase='1234567890', subcas_allowed=True )
        
        ## Intermediate CA object
        self.ica = CertificateAuthority( common_name='Intermediate CA', name='Intermediate_CA', description="unit test IM CA", country='DE', \
                                         state='Bavaria', locality='Berlin', organization='Bozo Clown Inc.', OU=None, email='a@b.com', valid_days=365, \
                                         key_length=1024, expiry_date='', created='', revoked=None, active=None, serial=None, ca_chain=None, \
                                         pem_encoded=True, der_encoded=False, parent=self.rca, parent_passphrase="1234567890", \
                                         passphrase='1234567890', subcas_allowed=True)
        
        ## Sub CA object (RootCA->IntermediateCA->SubCA)
        self.sca = CertificateAuthority( common_name='Sub CA', name='Sub_CA', description="unit test sub CA", country='DE', state='Bavaria', \
                                         locality='Munich', organization='Bozo Clown Inc.', OU='IT', email='a@b.com', valid_days=365, \
                                         key_length=1024, expiry_date='', created='', revoked=None, active=None, serial=None, ca_chain=None, \
                                         pem_encoded=True, der_encoded=False, parent=self.ica, parent_passphrase="1234567890", \
                                         passphrase='1234567890', subcas_allowed=False)
        
        
        openssl.refresh_pki_metadata([self.rca, self.ica, self.sca])
        self.rca_action = openssl.OpensslActions(self.rca)
        self.ica_action = openssl.OpensslActions(self.ica)
        self.sca_action = openssl.OpensslActions(self.sca)
    
    def test_001_OpensslExec(self):
        self.assertTrue(self.rca_action.exec_openssl(['version'], None))
    
    def test_002_GenerateKey(self):
        self.rca_action.generate_key()
        self.assertTrue(os.path.exists(self.rca_action.key))
    
    def test_003_GenerateCsr(self):
        self.rca_action.generate_csr()
        self.assertTrue(os.path.exists(self.rca_action.csr))
    
    def test_004_GenerateSelfSignedCA(self):
        self.rca_action.generate_self_signed_cert()
        self.assertTrue(os.path.exists(self.rca_action.crt))
    
    def test_005_DerExport(self):
        self.rca_action.generate_der_encoded()
        self.assertTrue(os.path.exists(self.rca_action.der))
    
    def test_006_DerRemove(self):
        self.rca_action.remove_der_encoded()
        self.assertFalse(os.path.exists(self.rca_action.der))
    
    def test_007_SubjectBuild(self):
        self.assertEqual(subject_for_object(self.rca), '/CN=%s/C=%s/ST=%s/localityName=%s/O=%s/organizationalUnitName=%s/emailAddress=%s' % \
                                                         ( self.rca.common_name, self.rca.country, self.rca.state,
                                                           self.rca.locality, self.rca.organization, self.rca.OU, self.rca.email ))
    
    def test_008_GenerateCrl(self):
        self.rca_action.generate_crl(ca=self.rca.name, pf='1234567890')
        self.assertTrue(os.path.exists(self.rca_action.crl))
    
    def test_009_SaveObject(self):
        self.assertEqual(self.rca.save(), None)
    
    def test_010_GetSerial(self):
        self.assertEqual(self.rca_action.get_serial_from_cert(), CertificateAuthority.objects.get(pk=1).serial)
    
    def test_011_RemoveObject(self):
        ca = CertificateAuthority.objects.get(pk=1)
        self.assertEqual(ca.delete(None), None)
    
    def test_012_ReSaveObject(self):
        self.assertEqual(self.rca.save(), None)
    
    def test_013_CreateIntermediateCA(self):       
        self.assertEqual(self.ica.save(), None)
    
    def test_014_CreateLeafCA(self):
        self.assertEqual(self.sca.save(), None)
    
    def test_015_RevokeIntermediateCA(self):
        self.ica.action = 'revoke'
        self.ica.parent_passphrase = self.rca.passphrase
        self.ica.pk = CertificateAuthority.objects.get(name=self.ica.name).pk
        self.assertEqual(self.ica.save(), None)
    
    def test_016_CheckCertificateRevokeStatus(self):
        self.ica.serial = CertificateAuthority.objects.get(name=self.ica.name).serial
        self.assertTrue(self.ica_action.get_revoke_status_from_cert())
    
    def test_017_FilesForObject(self):
        self.assertEqual(type(files_for_object(self.rca)), type(dict()))
    
class EmailDeliveryTestCase(unittest.TestCase):
    
    def setUp(self): pass
    
    def test_101_CheckSmtpConnection(self):
        if PKI_ENABLE_EMAIL:
            self.assertTrue(get_connection(backend="django.core.mail.backends.smtp.EmailBackend").open())
        else:
            self.assertTrue(True)

class HttpClientTestCase(TestCase):
    fixtures = ["test_users.json", ]
    
    def setUp(self):
        self.c = Client()
    
    def test_201_AddCertificateAuthority(self):
        self.c.login(username="pki_user_1", password="admin")
        r = self.c.post('/admin/pki/certificateauthority/add/', { 'common_name': 'Root CA', 'name': 'Root_CA', 'description': "unit test", \
                                                             'country': 'DE', 'state': 'Bavaria', 'locality': 'Munich', 'organization': 'Bozo Clown Inc.', \
                                                             'OU': 'IT', 'email': 'a@b.com', 'valid_days': 1000, 'key_length': 1024, 'expiry_date': '', \
                                                             'created': '', 'revoked': '', 'active': '', 'serial': '', 'ca_chain': '', \
                                                             'pem_encoded': True, 'der_encoded': False, 'parent': '', 'passphrase': '1234567890', \
                                                             'subcas_allowed': True
                                                           })
        
        self.failUnlessEqual(r.status_code, 200)
    
    def test_201_DownloadWithoutLogin(self):
        r = self.c.get('/admin/pki/download/ca/1')
        self.assertEqual(r.status_code, 404)
    
    def test_209_AdminLogin(self):
        self.assertTrue(self.c.login(username="pki_user_1", password="admin"))

class TestProjectWindmillTest(djangotest.WindmillDjangoUnitTest):
    fixtures = ["test_users.json"]
    test_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'windmilltests')
    browser  = 'firefox'



"""Graphviz support for django-pki"""

from pki.settings import PKI_ENABLE_GRAPHVIZ, PKI_GRAPHVIZ_DIRECTION

if PKI_ENABLE_GRAPHVIZ is True:
    try:
        import pygraphviz as pgv
    except ImportError, e:
        raise Exception( "Failed to import pygraphviz. Disable PKI_ENABLE_GRAPHVIZ or install pygraphviz: %s" % e )

from pki.models import Certificate, CertificateAuthority

##------------------------------------------------------------------##
## Graphviz functions
##------------------------------------------------------------------##

def ObjectChain(object, target):
    """Render object chain PNG.
    
    Render a graphviz image for the given object and save the resulting PNG
    in target.
    """
    
    ## Create graph object
    G = pgv.AGraph(directed=True, layout="dot", pad="0.2", rankdir="TB")
    
    ## Determine shape on object instance
    if isinstance( object, Certificate):
        o_shape = "note"
    elif isinstance( object, CertificateAuthority):
        o_shape = "folder"
    else:
        raise Exception( "Invalid object instance given!" )
    
    ## Set fill color bases in state
    if object.active:
        obj_fill = "green3"
    else:
        obj_fill = "red"
    
    edges = []
    
    ## Add given object to graph
    G.add_node(object.common_name, shape=o_shape, style="filled, bold", fillcolor=obj_fill, fontcolor="white")
    
    ## Get parents if any
    if object.parent != None:
        
        ## Set p to objects parent
        p = object.parent
        
        ## Add parent node to graph
        G.add_node(p.common_name, shape="folder", color="green3", style="bold")
        
        ## Set initial edge between requested onject and it's parent
        edges.append( [p.common_name, object.common_name] )
        
        while p != None:
            
            if p.active:
                col = "green3"
            else:
                col = "red"
            
            G.add_node(p.common_name, shape="folder", color=col, style="bold")
            
            if p.parent:
                edges.append( [p.parent.common_name, p.common_name] )
            
            p = p.parent
    
    ## Draw the edges
    for e in edges:
        G.add_edge( e[0], e[1] )
    
    G.layout()
    G.draw(target, format='png')
    
    return True

def ObjectTree(object, target):
    """Render object tree PNG.
    
    Render a graphviz image for the entire object tree object and save the resulting PNG
    in target.
    """
    
    ##-------------------------------------##
    ## Helper function for tree traversal
    ##-------------------------------------##
    def TraverseToBottom(r_id, graph=None):
        """Traverse the PKI tree down from a given id"""
        
        c = CertificateAuthority.objects.get(id=r_id)
        
        if c.subcas_allowed == True:
            x = CertificateAuthority.objects.filter(parent__id=c.pk)
            
            for ca in x:
                
                if graph != None:
                    if ca.active is True:
                        col = "green3"
                    else:
                        col = "red"
                    
                    graph.add_node(ca.common_name, shape='folder', color=col, style="bold")
                    graph.add_edge(c.common_name, ca.common_name, color="black", weight="4.5")
                
                if ca.subcas_allowed == True:
                    TraverseToBottom(ca.pk, graph)
                else:
                    certs = Certificate.objects.filter(parent__id=ca.pk)
                    
                    if certs:
                        subgraph_list = [ ca.common_name ]
                        
                        for cert in certs:
                            
                            subgraph_list.append( cert.common_name )
                            
                            if graph != None:
                                if cert.active:
                                    col = "green3"
                                else:
                                    col = "red"
                                
                                graph.add_node(str(cert.common_name), shape='note', color=col, style="bold")
                                graph.add_edge(ca.common_name, cert.common_name, color="black", weight="4.5")
                        
                        sg = graph.subgraph(nbunch=subgraph_list, name="cluster_%d" % ca.pk, style='bold', color='black', label="")
    
    ##-------------------------------------##
    ## Object tree starts here
    ##-------------------------------------##
    
    ## Create graph object
    G = pgv.AGraph(directed=True, layout="dot", pad="0.2", ranksep="1.00", nodesep="0.10", rankdir=PKI_GRAPHVIZ_DIRECTION,)
                   #label="PKI Tree of CertificateAuthority \"%s\"" % str(object.common_name), labeljust="l", labelloc="t")
    
    if object.active: obj_fill = "green3"
    else: obj_fill = "red"
    
    G.add_node(object.common_name, shape='folder', style="filled,bold", fillcolor=obj_fill, fontcolor="white")
    
    if not isinstance(object, CertificateAuthority):
        raise Exception( "Object has to be of type CertificateAuthority, not %s" % object.__class__.__name__ )
    
    ## Find top parents
    if object.parent != None:
        p = object.parent
        
        while p != None:
            if p.parent == None:
                if p.active:
                    col = "green3"
                else:
                    col = "red"
                
                G.add_node(p.common_name, shape='folder', color=col, style="bold")
                TraverseToBottom(p.id, G)
            p = p.parent
    else:
        TraverseToBottom(object.id, G)
    
    G.layout()
    G.draw(target, format='png')
    
    return True



import os
import re
import sys
import string
import random

from subprocess import Popen, PIPE, STDOUT
from shutil import rmtree
from logging import getLogger

from django.template.loader import render_to_string

from pki.settings import PKI_OPENSSL_BIN, PKI_OPENSSL_CONF, PKI_DIR, PKI_OPENSSL_TEMPLATE, PKI_SELF_SIGNED_SERIAL
from pki.helper import subject_for_object
import pki.models

try:
    # available in python-2.5 and greater
    from hashlib import md5 as md5_constructor
except ImportError:
    # compatibility fallback
    from md5 import new as md5_constructor

logger = getLogger("pki")

##------------------------------------------------------------------##
## OpenSSLConfig: Config related stuff
##------------------------------------------------------------------##

def refresh_pki_metadata(ca_list):
    """Refresh pki metadata (PKI storage directories and openssl configuration files)

    Each ca_list element is a dictionary:
    'name': CA name
    'subcas_allowed': sub CAs allowed (boolean)
    """
    
    status = True
    
    # refresh directory structure
    dirs = { 'certs'  : 0755,
             'private': 0700,
             'crl'    : 0755,
           }
    
    try:
        # create base PKI directory if necessary
        if not os.path.exists(PKI_DIR):
            logger.info('Creating base PKI directory')
            os.mkdir(PKI_DIR, 0700)
        
        # list of old CA directories for possible purging
        purge_dirs = set([os.path.join(PKI_DIR, d) for d in os.listdir(PKI_DIR)
                          if os.path.isdir(os.path.join(PKI_DIR, d))])
        
        # loop over CAs and create necessary filesystem objects
        for ca in ca_list:
            
            ca_dir = os.path.join(PKI_DIR, ca.name)
            
            # create CA directory if necessary
            if not ca_dir in purge_dirs:
                logger.info('Creating base directory for CA %s' % ca.name)
                os.mkdir(ca_dir)
                
                # create nested directories for key storage with proper permissions
                for d, m in dirs.items():
                    os.mkdir(os.path.join(ca_dir, d), m)
                
                initial_serial = 0x01
                
                try:
                    if not ca.parent and int(PKI_SELF_SIGNED_SERIAL) > 0:
                        initial_serial = PKI_SELF_SIGNED_SERIAL+1 
                except ValueError:
                    logger.error( "PKI_SELF_SIGNED_SERIAL failed conversion to int!" )
                
                h2s = '%X' % initial_serial
                
                if len(h2s) % 2 == 1:
                    h2s = '0' + h2s
                
                # initialize certificate serial number
                s = open(os.path.join(ca_dir, 'serial'), 'wb')
                s.write(h2s)
                s.close()
                
                # initialize CRL serial number
                s = open(os.path.join(ca_dir, 'crlnumber'), 'wb')
                s.write('01')
                s.close()
                
                # touch certificate index file
                open(os.path.join(ca_dir, 'index.txt'), 'wb').close()
            
            # do not delete existing CA dir
            purge_dirs.discard(ca_dir)
        
        # purge unused CA directories
        for d in purge_dirs:
            if os.path.isdir(d):
                # extra check in order to keep unrelated directory from recursive removal...
                # (in case if something wrong with paths)
                # probably can be removed when debugging will be finished
                if os.path.isfile(os.path.join(d, 'crlnumber')):
                    logger.debug("Purging CA directory tree %s" % d)
                    rmtree(d) # FIXME: commented for debugging purposes
                else:
                    logger.warning('Directory %s does not contain any metadata, preserving it' % d)
        
    except OSError, e: # FIXME: probably catch any exception here, not just OS
        status = False
        logger.error("Refreshing directory structure failed: %s" % e)
    
    # prepare context for template rendering
    ctx = {'ca_list': ca_list}

    # render template and save result to openssl.conf
    try:
        conf = render_to_string(PKI_OPENSSL_TEMPLATE, ctx)
        
        f = open(PKI_OPENSSL_CONF, 'wb')
        f.write(conf)
        f.close()

    except:
        raise Exception( 'Failed to render OpenSSL template' )
        status = False

    return status # is it used somewhere?

##------------------------------------------------------------------##
## OpenSSLActions: All non config related actions
##------------------------------------------------------------------##

class OpensslActions():
    """OpenSSL command and task wrapper class
    
    instance must be a CertificateAuthority or Certificate object.
    """
    
    def __init__(self, instance):
        """Initialize shared varaibles and verify instance type"""
        
        self.i    = instance
        self.subj = subject_for_object(self.i)
        
        if self.i.parent != None:
            self.parent_certs = os.path.join(PKI_DIR, self.i.parent.name, 'certs')
            self.crl = os.path.join(PKI_DIR, self.i.parent.name, 'crl', '%s.crl.pem' % self.i.parent.name)
        else:
            self.parent_certs = os.path.join(PKI_DIR, self.i.name, 'certs')
            self.crl = os.path.join(PKI_DIR, self.i.name, 'crl', '%s.crl.pem' % self.i.name)
        
        if isinstance(instance, pki.models.CertificateAuthority):
            ca_dir      = os.path.join(PKI_DIR, self.i.name)
            self.key    = os.path.join(ca_dir, 'private', '%s.key.pem' % self.i.name)
            self.ext    = ''
            self.pkcs12 = False
            self.i.subjaltname = ''
        elif isinstance(instance, pki.models.Certificate):
            ca_dir      = os.path.join(PKI_DIR, self.i.parent.name)
            self.key    = os.path.join(ca_dir, 'certs', '%s.key.pem' % self.i.name)
            self.ext    = '-extensions v3_cert'
            self.pkcs12 = os.path.join(ca_dir, 'certs', '%s.cert.p12' % self.i.name)
            
            if not self.i.subjaltname:
                self.i.subjaltname = 'email:copy'
        else:
            raise Exception( "Given object type is unknown!" )
        
        self.csr  = os.path.join(ca_dir, 'certs', '%s.csr.pem'  % self.i.name)
        self.crt  = os.path.join(ca_dir, 'certs', '%s.cert.pem' % self.i.name)
        self.der  = os.path.join(ca_dir, 'certs', '%s.cert.der' % self.i.name)
        
        ## Generate a random string as ENV variable name
        self.env_pw = "".join(random.sample(string.letters+string.digits, 10))
    
    def exec_openssl(self, command, env_vars=None):
        """Run a openssl command.
        
        command is prefixed with openssl binary from PKI_OPENSSL_BIN
        env_vars is a dict containing the set environment variables
        """
        
        c = [PKI_OPENSSL_BIN]
        c.extend(command)
        
        # add PKI_DIR environment variable if caller did not set it
        if env_vars:
            env_vars.setdefault('PKI_DIR', PKI_DIR)
        else:
            env_vars = { 'PKI_DIR': PKI_DIR }
        
        proc = Popen( c, shell=False, env=env_vars, stdin=PIPE, stdout=PIPE, stderr=STDOUT )
        stdout_value, stderr_value = proc.communicate()
        
        if proc.returncode != 0:
            logger.error( 'openssl command "%s" failed with returncode %d' % (c[1], proc.returncode) )
            logger.error( stdout_value )
            
            raise Exception( stdout_value )
        else:
            return stdout_value
    
    def generate_key(self):
        """RSA key generation.
        
        Key will be encrypted with des3 if passphrase is given.
        """
        
        key_type = po = pf = ''
        
        if self.i.passphrase:
            key_type = '-des3'
            po = '-passout'
            pf = 'env:%s' % self.env_pw
        
        command = 'genrsa %s -out %s %s %s %s' % (key_type, self.key, po, pf, self.i.key_length)
        self.exec_openssl(command.split(), env_vars={ self.env_pw: str(self.i.passphrase) } )
    
    def generate_self_signed_cert(self):
        """Generate a self signed root certificate.
        
        Serial is set to user specified value when PKI_SELF_SIGNED_SERIAL > 0
        """
        
        logger.info( 'Generating self-signed root certificate' )
        
        command = ['req', '-config', PKI_OPENSSL_CONF, '-verbose', '-batch', '-new', '-x509', '-subj', self.subj, '-days', str(self.i.valid_days), \
                   '-extensions', 'v3_ca', '-key', self.key, '-out', self.crt, '-passin', 'env:%s' % self.env_pw]
        
        try:
            if PKI_SELF_SIGNED_SERIAL and int(PKI_SELF_SIGNED_SERIAL) > 0:
                command.extend( [ '-set_serial', str(PKI_SELF_SIGNED_SERIAL) ] )
        except ValueError, e:
            logger.error( "Not setting inital serial number to %s. Fallback to random number" % PKI_SELF_SIGNED_SERIAL )
            logger.error( e )
        
        self.exec_openssl( command, env_vars={ self.env_pw: str(self.i.passphrase), })
    
    def generate_csr(self):
        """CSR (Certificate Signing Request) generation"""
        
        logger.info( 'Generating the CSR for %s' % self.i.name )
        
        command = ['req', '-config', PKI_OPENSSL_CONF, '-new', '-batch', '-subj', self.subj, '-key', self.key, '-out', self.csr, \
                   '-days', str(self.i.valid_days), '-passin', 'env:%s' % self.env_pw]        
        self.exec_openssl(command, env_vars={ self.env_pw: str(self.i.passphrase) })
    
    def generate_der_encoded(self):
        """Generate a DER encoded certificate"""
        
        logger.info( 'Generating DER encoded certificate for %s' % self.i.name )
        
        command = 'x509 -in %s -out %s -outform DER' % (self.crt, self.der)
        self.exec_openssl(command.split())
    
    def remove_der_encoded(self):
        """Remove a DER encoded certificate"""
        
        if os.path.exists(self.der):
            logger.info( 'Removal of DER encoded certificate for %s' % self.i.name )
            
            try:
                os.remove(self.der)
            except OSError, e:
                logger.error( "Failed to remove %s" % self.der )
                raise Exception( e )
    
    def generate_pkcs12_encoded(self):
        """Generate a PKCS12 encoded certificate.
        
        Passphrase is required as empty passwords not work in batch mode.
        """
        
        command = 'pkcs12 -export -nokeys -in %s -inkey %s -out %s -passout env:%s' % (self.crt, self.key, self.pkcs12, self.env_pw)
        self.exec_openssl(command.split(), env_vars={ self.env_pw: str(self.i.pkcs12_passphrase) })
    
    def remove_pkcs12_encoded(self):
        """Remove a PKCS12 encoded certificate if it exists"""
        
        if self.pkcs12 and os.path.exists(self.pkcs12):
            logger.info( 'Removal of PKCS12 encoded certificate for %s' % self.i.name )
            
            os.remove(self.pkcs12)
    
    def remove_complete_certificate(self):
        """Remove all files related to the given certificate.
        
        This includes the hash alias, key, csr and the certificate itself.
        """
        
        self.remove_der_encoded()
        self.remove_pkcs12_encoded()
        
        hash = "%s/%s.0" % (self.parent_certs, self.get_hash_from_cert())
        if os.path.exists(hash):
            os.remove(hash)
        
        serial = "%s/%s.pem" % (self.parent_certs, self.get_serial_from_cert())
        if os.path.exists(serial):
            os.remove(serial)
        
        if os.path.exists(self.csr):
            os.remove(self.csr)
        
        if os.path.exists(self.key):
            os.remove(self.key)
        
        if os.path.exists(self.crt):
            os.remove(self.crt)
    
    def sign_csr(self):
        """Sign the CSR.
        
        Certificate signing and hash creation in CA's certificate directory
        """
        
        try:
            extension = "-extensions %s" % self.i.cert_extension
        except:
            extension = ""
        
        command = 'ca -config %s -name %s -batch %s -in %s -out %s -days %d %s -passin env:%s' % \
                  ( PKI_OPENSSL_CONF, self.i.parent, self.ext, self.csr, self.crt, self.i.valid_days, extension, self.env_pw)
        self.exec_openssl(command.split(), env_vars={ self.env_pw: str(self.i.parent_passphrase), "S_A_N": self.i.subjaltname, })
        
        ## Get the just created serial
        if self.parent_certs:
            serial = self.get_serial_from_cert()
            hash   = self.get_hash_from_cert()
            
            if os.path.exists('%s/%s.0' % (self.parent_certs, hash)):
                os.remove('%s/%s.0' % (self.parent_certs, hash))
            
            os.symlink('%s.pem' % serial, '%s/%s.0' % (self.parent_certs, hash))
    
    def revoke_certificate(self, ppf):
        """Revoke a certificate.
        
        Requires the parents passphrase.
        """
        
        ## Check if certificate is already revoked. May have happened during a incomplete transaction
        if self.get_revoke_status_from_cert():
            logger.info( "Skipping revoke as it already happened" )
            return True
        
        command = 'ca -config %s -name %s -batch -revoke %s -passin env:%s' % (PKI_OPENSSL_CONF, self.i.parent, self.crt, self.env_pw)
        self.exec_openssl(command.split(), env_vars={ self.env_pw: str(ppf) })
    
    def renew_certificate(self):
        """Renew/Reissue a given certificate.
        
        This requires the CSR to be available.
        """
        
        logger.info( 'Renewing certificate %s' % self.i.name )
        
        if os.path.exists(self.csr):
            self.sign_csr()
        else:
            raise Exception( "Failed to renew certificate %s! CSR is missing!" % self.i.name )
    
    def generate_crl(self, ca=None, pf=None):
        """CRL (Certificate Revocation List) generation.
        
        Requires the Certificate Authority and the passphrase. 
        """
        
        crl = os.path.join(PKI_DIR, ca, 'crl', '%s.crl.pem' % ca)
        
        command = 'ca -config %s -name %s -gencrl -out %s -crldays 1 -passin env:%s' % (PKI_OPENSSL_CONF, ca, crl, self.env_pw)
        self.exec_openssl(command.split(), env_vars={ self.env_pw: str(pf) })
    
    def update_ca_chain_file(self):
        """Build/update the CA chain.
        
        Generates a chain file containing all CA's required to verify the given certificate.
        """
        
        ## Build list of parents
        chain = []
        chain_str = ''
        
        p = self.i.parent
        
        if self.i.parent == None:
            chain.append( self.i.name )
        else:
            chain.append( self.i.name )
            while p != None:
                chain.append(p.name)
                p = p.parent
        
        chain.reverse()
        
        chain_file = os.path.join( PKI_DIR, self.i.name, '%s-chain.cert.pem' % self.i.name )
        
        try:
            w = open(chain_file, 'w')
            
            for c in chain:
                cert_file = os.path.join( PKI_DIR, c, 'certs', '%s.cert.pem' % c )
                command = 'x509 -in %s' % cert_file
                output  = self.exec_openssl(command.split())
                
                ## Get the subject to print it first in the chain file
                subj = subject_for_object(self.i)
                
                w.write( '%s\n' % subj )
                w.write(output)
            
            w.close()
        except:
            raise Exception( 'Failed to write chain file!' )
    
    def get_serial_from_cert(self):
        """Extract serial from certificate.
        
        Use openssl to get the serial number from a certificate.
        """
        
        command = 'x509 -in %s -noout -serial' % self.crt
        output  = self.exec_openssl(command.split())
        
        x = output.rstrip("\n").split('=')
        
        return x[1]
    
    def get_hash_from_cert(self):
        """Extract hash from certificate.
        
        Use openssl to get the hash value from a certificate.
        """
        
        command = 'x509 -hash -noout -in %s' % self.crt
        output  = self.exec_openssl(command.split())
        
        return output.rstrip("\n")
    
    def get_revoke_status_from_cert(self):
        """Get the revoke status from certificate.
        
        Certificate is revoked => True
        Certificate is active  => False
        """
        
        command = 'crl -text -noout -in %s' % self.crl
        output  = self.exec_openssl(command.split())
        
        serial_re = re.compile('^\s+Serial\sNumber\:\s+(\w+)')
        lines = output.split('\n')
        
        for l in lines:
            if serial_re.match(l):
                if serial_re.match(l).group(1) == self.i.serial:
                    logger.info( "The certificate is revoked" )
                    return True
        
        return False



# Generated by the windmill services transformer
from windmill.authoring import WindmillTestClient

def test_AddRootCA():
    client = WindmillTestClient(__name__)

    client.open(url=u'http://127.0.0.1:8000/admin/')
    client.waits.forPageLoad(timeout=u'8000')
    client.type(text=u'admin', id=u'id_username')
    client.type(text=u'admin', id=u'id_password')
    client.click(value=u'Log in')
    client.waits.forPageLoad(timeout=u'20000')
    client.waits.forElement(xpath=u"//div[@id='content-main']/div[2]/table/tbody/tr[1]/td[1]/a", timeout=u'8000')
    client.click(xpath=u"//div[@id='content-main']/div[2]/table/tbody/tr[1]/td[1]/a")
    client.waits.forPageLoad(timeout=u'20000')
    client.waits.forElement(timeout=u'8000', id=u'id_description')
    client.click(id=u'id_description')
    client.type(text=u'Windmill RootCA', id=u'id_description')
    client.click(id=u'id_common_name')
    client.type(text=u'Root CA', id=u'id_common_name')
    client.click(id=u'id_state')
    client.type(text=u'Bavaria', id=u'id_state')
    client.type(text=u'Munich', id=u'id_locality')
    client.click(id=u'id_organization')
    client.type(text=u'My Company', id=u'id_organization')
    client.click(id=u'id_email')
    client.type(text=u'daniel@linuxaddicted.de', id=u'id_email')
    client.click(id=u'id_valid_days')
    client.type(text=u'365', id=u'id_valid_days')
    client.click(id=u'id_passphrase')
    client.type(text=u'1234567890', id=u'id_passphrase')
    client.check(id=u'id_der_encoded')
    client.check(id=u'id_subcas_allowed')
    client.click(xpath=u"//form[@id='certificateauthority_form']/div[2]/div[2]/input[1]")
    client.waits.forPageLoad(timeout=u'20000')
    client.asserts.assertTextIn(xpath=u"//div[@id='container']/ul/li", validator=u'The certificate authority "Root_CA" was added successfully.')




import os
import logging

from pki.settings import PKI_ENABLE_EMAIL

if PKI_ENABLE_EMAIL is True:
    try:
        from django.core.mail import EmailMessage
        import zipfile
    except ImportError, e:
        raise Exception( "Library import failed. Disable PKI_ENABLE_EMAIL or install/update the missing Python lib: %s" % e )

from django.shortcuts import get_object_or_404

from pki.models import Certificate, CertificateAuthority
from pki.helper import files_for_object, subject_for_object, build_zip_for_object

logger = logging.getLogger("pki")

##------------------------------------------------------------------##
## Email functions
##------------------------------------------------------------------##

def SendCertificateData(obj, request):
    """Send the zipped certificate data as email.
    
    Verify that the given object has all the flags set, create a zipfile and mail it to the
    email address from the certificate.
    """
    
    ## Check that email flag is set in the DB
    if obj.email:
        zip_f = build_zip_for_object(obj, request)
        
        ## Read ZIP content and remove it
        try:
            if os.path.exists(zip_f):
                f = open(zip_f)
                x = f.read()
                f.close()
                
                os.remove(zip_f)
        except OSError,e:
            logger.error( "Failed to read zipfile: %s" % e)
            raise Exception( e )
        
        ## Build email obj and send it out
        parent_name = 'self-signed'
        if obj.parent:
            parent_name = obj.parent.common_name
        
        subj_msg = subject_for_object(obj)
        body_msg = "Certificate data sent by django-pki:\n\n  * subject: %s\n  * parent: %s\n" % (subj_msg, parent_name)
        
        email = EmailMessage( to=[obj.email,], subject="Certificate data for \"%s\"" % subj_msg, body=body_msg,  )
        email.attach( 'PKI_DATA_%s.zip' % obj.name, x, 'application/zip' )
        email.send(fail_silently=False)



import os
import datetime
from logging import getLogger
from shutil import rmtree

from django.db import models

from pki.openssl import OpensslActions, md5_constructor, refresh_pki_metadata
from pki.settings import ADMIN_MEDIA_PREFIX, MEDIA_URL, PKI_BASE_URL, PKI_DEFAULT_COUNTRY, PKI_ENABLE_GRAPHVIZ, PKI_ENABLE_EMAIL

logger = getLogger("pki")

##------------------------------------------------------------------##
## Choices
##------------------------------------------------------------------##

KEY_LENGTH = ( (1024, '1024'), (2048, '2048'), (4096, '4096'), )
POLICY     = ( ('policy_match', 'policy_match'), ('policy_anything', 'policy_anything'), )
ACTIONS    = ( ('create', 'create'), ('update', 'update'), ('revoke', 'revoke'), ('renew', 'renew'), )
CA_TYPES   = ( ('RootCA', 'self-signed (RootCA)'), ('SubCA', 'SubCA'), )
EXTENSIONS = ( ('v3_server_cert', 'V3 Server'), ('v3_client_cert', 'V3 Client' ), )
COUNTRY    = ( ('AD', 'AD'),('AE', 'AE'),('AF', 'AF'),('AG', 'AG'),('AI', 'AI'),('AL', 'AL'),('AM', 'AM'),
               ('AN', 'AN'),('AO', 'AO'),('AQ', 'AQ'),('AR', 'AR'),('AS', 'AS'),('AT', 'AT'),('AU', 'AU'),
               ('AW', 'AW'),('AZ', 'AZ'),('BA', 'BA'),('BB', 'BB'),('BD', 'BD'),('BE', 'BE'),('BF', 'BF'),
               ('BG', 'BG'),('BH', 'BH'),('BI', 'BI'),('BJ', 'BJ'),('BM', 'BM'),('BN', 'BN'),('BO', 'BO'),
               ('BR', 'BR'),('BS', 'BS'),('BT', 'BT'),('BU', 'BU'),('BV', 'BV'),('BW', 'BW'),('BY', 'BY'),
               ('BZ', 'BZ'),('CA', 'CA'),('CC', 'CC'),('CF', 'CF'),('CG', 'CG'),('CH', 'CH'),('CI', 'CI'),
               ('CK', 'CK'),('CL', 'CL'),('CM', 'CM'),('CN', 'CN'),('CO', 'CO'),('CR', 'CR'),('CS', 'CS'),
               ('CU', 'CU'),('CV', 'CV'),('CX', 'CX'),('CY', 'CY'),('CZ', 'CZ'),('DD', 'DD'),('DE', 'DE'),
               ('DJ', 'DJ'),('DK', 'DK'),('DM', 'DM'),('DO', 'DO'),('DZ', 'DZ'),('EC', 'EC'),('EE', 'EE'),
               ('EG', 'EG'),('EH', 'EH'),('ER', 'ER'),('ES', 'ES'),('ET', 'ET'),('FI', 'FI'),('FJ', 'FJ'),
               ('FK', 'FK'),('FM', 'FM'),('FO', 'FO'),('FR', 'FR'),('FX', 'FX'),('GA', 'GA'),('GB', 'GB'),
               ('GD', 'GD'),('GE', 'GE'),('GF', 'GF'),('GH', 'GH'),('GI', 'GI'),('GL', 'GL'),('GM', 'GM'),
               ('GN', 'GN'),('GP', 'GP'),('GQ', 'GQ'),('GR', 'GR'),('GS', 'GS'),('GT', 'GT'),('GU', 'GU'),
               ('GW', 'GW'),('GY', 'GY'),('HK', 'HK'),('HM', 'HM'),('HN', 'HN'),('HR', 'HR'),('HT', 'HT'),
               ('HU', 'HU'),('ID', 'ID'),('IE', 'IE'),('IL', 'IL'),('IN', 'IN'),('IO', 'IO'),('IQ', 'IQ'),
               ('IR', 'IR'),('IS', 'IS'),('IT', 'IT'),('JM', 'JM'),('JO', 'JO'),('JP', 'JP'),('KE', 'KE'),
               ('KG', 'KG'),('KH', 'KH'),('KI', 'KI'),('KM', 'KM'),('KN', 'KN'),('KP', 'KP'),('KR', 'KR'),
               ('KW', 'KW'),('KY', 'KY'),('KZ', 'KZ'),('LA', 'LA'),('LB', 'LB'),('LC', 'LC'),('LI', 'LI'),
               ('LK', 'LK'),('LR', 'LR'),('LS', 'LS'),('LT', 'LT'),('LU', 'LU'),('LV', 'LV'),('LY', 'LY'),
               ('MA', 'MA'),('MC', 'MC'),('MD', 'MD'),('MG', 'MG'),('MH', 'MH'),('ML', 'ML'),('MM', 'MM'),
               ('MN', 'MN'),('MO', 'MO'),('MP', 'MP'),('MQ', 'MQ'),('MR', 'MR'),('MS', 'MS'),('MT', 'MT'),
               ('MU', 'MU'),('MV', 'MV'),('MW', 'MW'),('MX', 'MX'),('MY', 'MY'),('MZ', 'MZ'),('NA', 'NA'),
               ('NC', 'NC'),('NE', 'NE'),('NF', 'NF'),('NG', 'NG'),('NI', 'NI'),('NL', 'NL'),('NO', 'NO'),
               ('NP', 'NP'),('NR', 'NR'),('NT', 'NT'),('NU', 'NU'),('NZ', 'NZ'),('OM', 'OM'),('PA', 'PA'),
               ('PE', 'PE'),('PF', 'PF'),('PG', 'PG'),('PH', 'PH'),('PK', 'PK'),('PL', 'PL'),('PM', 'PM'),
               ('PN', 'PN'),('PR', 'PR'),('PT', 'PT'),('PW', 'PW'),('PY', 'PY'),('QA', 'QA'),('RE', 'RE'),
               ('RO', 'RO'),('RU', 'RU'),('RW', 'RW'),('SA', 'SA'),('SB', 'SB'),('SC', 'SC'),('SD', 'SD'),
               ('SE', 'SE'),('SG', 'SG'),('SH', 'SH'),('SI', 'SI'),('SJ', 'SJ'),('SK', 'SK'),('SL', 'SL'),
               ('SM', 'SM'),('SN', 'SN'),('SO', 'SO'),('SR', 'SR'),('ST', 'ST'),('SU', 'SU'),('SV', 'SV'),
               ('SY', 'SY'),('SZ', 'SZ'),('TC', 'TC'),('TD', 'TD'),('TF', 'TF'),('TG', 'TG'),('TH', 'TH'),
               ('TJ', 'TJ'),('TK', 'TK'),('TM', 'TM'),('TN', 'TN'),('TO', 'TO'),('TP', 'TP'),('TR', 'TR'),
               ('TT', 'TT'),('TV', 'TV'),('TW', 'TW'),('TZ', 'TZ'),('UA', 'UA'),('UG', 'UG'),('UM', 'UM'),
               ('US', 'US'),('UY', 'UY'),('UZ', 'UZ'),('VA', 'VA'),('VC', 'VC'),('VE', 'VE'),('VG', 'VG'),
               ('VI', 'VI'),('VN', 'VN'),('VU', 'VU'),('WF', 'WF'),('WS', 'WS'),('YD', 'YD'),('YE', 'YE'),
               ('YT', 'YT'),('YU', 'YU'),('ZA', 'ZA'),('ZM', 'ZM'),('ZR', 'ZR'),('ZW', 'ZW'),('ZZ', 'ZZ'),
               ('ZZ', 'ZZ'),
              )

##------------------------------------------------------------------##
## Base DB classes
##------------------------------------------------------------------##

class CertificateBase(models.Model):
    """Base class for all type of certificates"""
    
    description  = models.CharField(max_length=255)
    country      = models.CharField(max_length=2, choices=COUNTRY, default='%s' % PKI_DEFAULT_COUNTRY.upper() )
    state        = models.CharField(max_length=32)
    locality     = models.CharField(max_length=32)
    organization = models.CharField(max_length=64)
    OU           = models.CharField(max_length=64,blank=True, null=True)
    email        = models.EmailField(blank=True, null=True)
    valid_days   = models.IntegerField()
    key_length   = models.IntegerField(choices=KEY_LENGTH, default=2048)
    expiry_date  = models.DateField(blank=True,null=True)
    created      = models.DateTimeField(blank=True,null=True)
    revoked      = models.DateTimeField(blank=True,null=True)
    active       = models.BooleanField(default=True, help_text="Turn off to revoke this certificate")
    serial       = models.CharField(max_length=64, blank=True, null=True)
    ca_chain     = models.CharField(max_length=200, blank=True, null=True)
    pem_encoded  = models.BooleanField(default=False)
    der_encoded  = models.BooleanField(default=False, verbose_name="Create DER encoded certificate (additional)")
    action       = models.CharField(max_length=32, choices=ACTIONS, default='create', help_text="create: Create (CA) certificate<br /> \
                                    update: Change description and export options<br /> \
                                    revoke: Revoke (CA) certificate. May break whole chain<br /> \
                                    renew: Re-sign csr. Key is unchanged. Chain stays valid<br /><br /> \
                                    Yellow fields can/have to be modified!")
    
    class Meta:
        abstract = True
    
    ##------------------------------------------------------------------##
    ## Helper functions
    ##------------------------------------------------------------------##
    
    def get_icon_html(self, state):
        """Return HTML based on state.
        
        True : Return Django's yes icon
        False: Return Django's no icon
        """
        
        if state is True:
            return '<center><img src="%simg/admin/icon-yes.gif" alt="True" /></center>' % ADMIN_MEDIA_PREFIX
        else:
            return '<center><img src="%simg/admin/icon-no.gif" alt="False" /></center>'  % ADMIN_MEDIA_PREFIX
    
    def get_pki_icon_html(self, img, alt="", title=""):
        """Return HTML for given image.
        
        Can add optional alt and title parameters.
        """
        
        img_path = os.path.join(PKI_BASE_URL, MEDIA_URL, 'pki/img', img)
        return '<img src="%s" alt="%s" title="%s"/>' % (img_path, alt, title)
    
    ##------------------------------------------------------------------##
    ## Changelist list_display functions
    ##------------------------------------------------------------------##
    
    def active_center(self):
        """Overwrite the Booleanfield admin for admin's changelist"""
        
        return self.get_icon_html(self.active)
    
    active_center.allow_tags = True
    active_center.short_description = 'Active'
    active_center.admin_order_field = 'active'
    
    def Serial_align_right(self):
        """Make serial in changelist right justified"""
        
        return '<div style="text-align:right;">%s</div>' % self.serial
    
    Serial_align_right.allow_tags = True
    Serial_align_right.short_description = 'Serial'
    Serial_align_right.admin_order_field = 'serial'
    
    def Description(self):
        """Limit description for changelist.
        
        Limit description to 30 characters to make changelist stay on one line per item.
        At least in most cases.
        """
        
        if len(self.description) > 30:
            return "%s..." % self.description[:30]
        else:
            return "%s" % self.description
    
    Description.allow_tags = True
    Description.admin_order_field = 'description'
    
    def Expiry_date(self):
        """Return expiry date with days left.
        
        Return color marked expiry date based on days left.
        < 0 : EXPIRED text and red font color
        < 30: Orange color
        > 30: Just date and days left
        """
        now = datetime.datetime.now().date()        
        diff = self.expiry_date - now
        
        if diff.days < 30 and diff.days >= 0:
            return '<font color="#f19d09"><strong>%s (%sd)</strong></font>' % (self.expiry_date, diff.days)
        elif diff.days < 0:
            return '<font color="red"><strong>%s (EXPIRED)</strong></font>' % self.expiry_date
        else:
            return '%s (%sd)' % (self.expiry_date, diff.days)
    
    Expiry_date.allow_tags = True
    Expiry_date.admin_order_field = 'expiry_date'
    
    def Chain_link(self):
        """Display chain link.
        
        If PKI_ENABLE_GRAPHVIZ is True a colored chain link is displayed. Otherwise
        a b/w chain icon without link is displayed.
        """
        
        type = "cert"
        
        if self.__class__.__name__ == "CertificateAuthority":
            type = "ca"
        
        if PKI_ENABLE_GRAPHVIZ:
            return '<center><a href="%s/pki/chain/%s/%d" target="_blank">%s</a></center>' % (PKI_BASE_URL, type, self.pk, self.get_pki_icon_html('chain.png', "Show chain", "Show object chain"))
        else:
            return '<center>%s</center>' % self.get_pki_icon_html("chain.png", "Show chain", "Enable setting PKI_ENABLE_GRAPHVIZ")
    
    Chain_link.allow_tags = True
    Chain_link.short_description = 'Chain'
    
    def Email_link(self):
        """Display email link based on status.
        
        If PKI_ENABLE_EMAIL or certificate isn't active a disabled (b/w) icon is displayed.
        If no email address is set in the certificate a icon with exclamation mark is displayed.
        Otherwise the normal icon is returned.
        """
        
        if not PKI_ENABLE_EMAIL:
            result  = '<center>%s</center>' % self.get_pki_icon_html("mail--arrow_bw.png", "Send email", "Enable setting PKI_ENABLE_EMAIL")
        elif not self.active:
            result  = '<center>%s</center>' % self.get_pki_icon_html("mail--arrow_bw.png", "Send email", "Certificate is revoked. Disabled")
        else:
            type = "cert"
            
            if self.__class__.__name__ == "CertificateAuthority": type = "ca"
            
            if self.email:
                result  = '<center><a href="%s/pki/email/%s/%d">%s</a></center>' % (PKI_BASE_URL, type, self.pk, self.get_pki_icon_html("mail--arrow.png", "Send email", "Send cert to specified email"))
            else:
                result  = '<center>%s</center>' % self.get_pki_icon_html("mail--exclamation.png", "Send email", "Certificate has no email set. Disabled")
        
        return result
    
    Email_link.allow_tags = True
    Email_link.short_description = 'Email'
    
    def Download_link(self):
        """Return a download icon.
        
        Based on object status => clickable icon or just a b/w image
        """
        
        if self.active:
            type = "cert"
            
            if self.__class__.__name__ == "CertificateAuthority": type = "ca"
            
            return '<center><a href="%s/pki/download/%s/%d/">%s ZIP</href></center></center>' % (PKI_BASE_URL, type, self.pk, \
                                                                                                 self.get_pki_icon_html("drive-download.png", "Download", "Download certificate data"))
        else:
            return '<center>%s <font color="grey">ZIP</font></center>' % self.get_pki_icon_html("drive-download_bw.png", "Download", "Cannot download because certificate is revoked")
    
    Download_link.allow_tags = True
    Download_link.short_description = 'Download'
    
    def Parent_link(self):
        """Return parent name.
        
        Returns parent's name when parent != None or self-signed
        """
        
        if self.parent:
            return '<a href="../certificateauthority/%d/" style="font-weight: bold;">%s</a>' % (self.parent.pk, self.parent.common_name)
        else:
            return "self-signed"
    
    Parent_link.allow_tags = True
    Parent_link.short_description = 'Parent'
    
##------------------------------------------------------------------##
## Certificate authority class
##------------------------------------------------------------------##

class CertificateAuthority(CertificateBase):
    """Certificate Authority model"""
    
    ##---------------------------------##
    ## Model definition
    ##---------------------------------##
    
    common_name       = models.CharField(max_length=64, unique=True)
    name              = models.CharField(max_length=64, unique=True, help_text="Only change the suggestion if you really know what you're doing")
    subcas_allowed    = models.BooleanField(verbose_name="Sub CA's allowed", help_text="If enabled you cannot sign certificates with this CA")
    parent            = models.ForeignKey('self', blank=True, null=True)
    type              = models.CharField(max_length=32, null=True, choices=CA_TYPES, default='RootCA')
    passphrase        = models.CharField(max_length=255, blank=True, help_text="At least 8 characters. Remeber this passphrase - <font color='red'> \
                                                                    <strong>IT'S NOT RECOVERABLE</strong></font><br>Will be shown as md5 encrypted string")
    parent_passphrase = models.CharField(max_length=255, null=True, blank=True, help_text="Leave empty if this is a top-level CA")
    pf_encrypted      = models.NullBooleanField()
    policy            = models.CharField(max_length=50, choices=POLICY, default='policy_anything', help_text='policy_match: All subject settings must \
                                                                                                              match the signing CA<br> \
                                                                                                              policy_anything: Nothing has to match the \
                                                                                                              signing CA')
    #crl_distribution  = models.URLField(verbose_name='CRL Distribution Point', null=True, blank=True, verify_exists=False, help_text='Optional CRL distribution URL (http://my.host.com/ca.crl)')

    class Meta:
        verbose_name_plural = 'Certificate Authorities'
        permissions         = ( ( "can_download_ca", "Can download", ), )
    
    def __unicode__(self):
        return self.name
    
    ##---------------------------------##
    ## Redefined functions
    ##---------------------------------##
    
    def save(self, force_insert=False, force_update=False):
        """Save the CertificateAuthority object"""
        
        if self.pk:
            ### existing CA
            if self.action in ('update', 'revoke', 'renew'):
                
                action = OpensslActions(self)
                prev   = CertificateAuthority.objects.get(pk=self.pk)
                
                if self.action == 'update':
                    
                    ## Create or remove DER certificate
                    if self.der_encoded:
                        action.generate_der_encoded()
                    else:
                        action.remove_der_encoded()
                    
                    prev.description = self.description
                    prev.der_encoded = self.der_encoded
                    
                elif self.action == 'revoke':
                    
                    ## DB-revoke all related certs
                    garbage = []
                    id_dict = { 'cert': [], 'ca': [], }
                    
                    from pki.views import chain_recursion as r_chain_recursion
                    r_chain_recursion(self.id, garbage, id_dict)
                    
                    for i in id_dict['cert']:
                        x = Certificate.objects.get(pk=i)
                        x.active         = False
                        x.der_encoded    = False
                        x.pem_encoded    = False
                        x.pkcs12_encoded = False
                        x.revoked        = datetime.datetime.now()
                        
                        super(Certificate, x).save()
                    
                    for i in id_dict['ca']:
                        x = CertificateAuthority.objects.get(pk=i)
                        x.active       = False
                        x.der_encoded  = False
                        x.pem_encoded  = False
                        x.revoked      = datetime.datetime.now()
                        
                        super(CertificateAuthority, x).save()
                    
                    ## Revoke and generate CRL
                    action.revoke_certificate(self.parent_passphrase)
                    action.generate_crl(self.parent.name, self.parent_passphrase)
                    
                    ## Modify fields
                    prev.parent_passphrase = None
                    prev.active            = False
                    prev.der_encoded       = False
                    prev.pem_encoded       = False
                    prev.revoked = datetime.datetime.now()
                    
                elif self.action == 'renew':
                    
                    ## Revoke if certificate is active
                    if self.parent and not action.get_revoke_status_from_cert():
                        action.revoke_certificate(self.parent_passphrase)
                        action.generate_crl(self.parent.name, self.parent_passphrase)
                    
                    ## Rebuild the ca metadata
                    self.rebuild_ca_metadata(modify=True, task='replace')
                    
                    ## Renew certificate and update CRL
                    if self.parent == None:
                        action.generate_self_signed_cert()
                        action.generate_crl(self.name, self.passphrase)
                    else:
                        action.renew_certificate()
                        action.generate_crl(self.parent.name, self.parent_passphrase)
                    
                    action.update_ca_chain_file()
                    
                    ## Modify fields
                    prev.created = datetime.datetime.now()
                    delta = datetime.timedelta(self.valid_days)
                    prev.expiry_date = datetime.datetime.now() + delta
                    prev.valid_days = self.valid_days
                    
                    prev.parent_passphrase = None
                    prev.active            = True
                    prev.pem_encoded       = True
                    prev.der_encoded       = self.der_encoded
                    prev.revoked           = None
                    
                    ## Get the new serial
                    prev.serial     = action.get_serial_from_cert()
                    #prev.passphrase = md5_constructor(self.passphrase).hexdigest()
                
                ## Save the data
                self = prev
                self.action = 'update'
                
                super(CertificateAuthority, self).save()
            else:
                
                raise Exception( 'Invalid action %s supplied' % self.action )
        else:
            ## Set creation data
            self.created = datetime.datetime.now()
            delta = datetime.timedelta(self.valid_days)
            self.expiry_date = datetime.datetime.now() + delta
            
            ## Force instance to be active
            self.active = True
            
            ## Reset the action
            self.action = 'update'
            
            ## Rebuild the ca metadata
            self.rebuild_ca_metadata(modify=True, task='append')
            
            ## Generate keys and certificates
            action = OpensslActions(self)
            action.generate_key()
            
            if not self.parent:
                action.generate_self_signed_cert()
            else:
                action.generate_csr()
                action.sign_csr()
            
            if self.der_encoded:
                action.generate_der_encoded()
            
            ## Generate CRL
            action.generate_crl(self.name, self.passphrase)
            
            ## Always enable pem encoded flag
            self.pem_encoded = True
            
            ## Get the serial from certificate
            self.serial = action.get_serial_from_cert()
            
            ## Generate ca chain (db field and chain file)
            chain = []
            chain_str = ''
            
            p = self.parent
            
            if self.parent == None:
                chain.append('self-signed')
            else:
                chain.append( self.common_name )
                while p != None:
                    chain.append(p.common_name)
                    p = p.parent
            
            chain.reverse()
            
            ## Build chain string and file
            for i in chain:
                if chain_str == '':
                    chain_str += '%s' % i
                else:
                    chain_str += ' &rarr; %s' % i
            
            self.ca_chain = chain_str
            
            action.update_ca_chain_file()
            
            ## Encrypt passphrase and blank parent's passphrase
            self.passphrase = md5_constructor(self.passphrase).hexdigest()
            self.parent_passphrase = None
            
        ## Save the data
        super(CertificateAuthority, self).save()
    
    def delete(self, passphrase):
        """Delete the CertificateAuthority object"""
        
        logger.info( "Certificate %s is going to be deleted" % self.name )
        
        ## Container for CA folders to delete
        self.remove_chain = []
        
        ## Is a revoke required?
        revoke_required = True
        
        ## Helper function for recusion
        def chain_recursion(r_id):
            
            ca = CertificateAuthority.objects.get(pk=r_id)
            self.remove_chain.append(ca.name)
            
            ## Search for related CA's
            child_cas = CertificateAuthority.objects.filter(parent=r_id)
            if child_cas:
                for ca in child_cas:
                    chain_recursion(ca.pk)
        
        if not self.parent:
            logger.info( "No revoking of certitifcates. %s is a toplevel CA" % self.name )
            revoke_required = False
        else:
            ## Collect child CA's and certificates
            chain_recursion(self.pk)
            logger.info( "Full chain is %s and pf is %s" % (self.remove_chain, self.passphrase))
        
        ## Remoke first ca in the chain
        if revoke_required:
            c_action = OpensslActions(CertificateAuthority.objects.get(pk=self.pk))
            c_action.revoke_certificate(passphrase)
            c_action.generate_crl(ca=self.parent.name, pf=passphrase)
        
        ## Rebuild the ca metadata
        self.rebuild_ca_metadata(modify=True, task='exclude')
        
        ## Call the "real" delete function
        super(CertificateAuthority, self).delete()
    
    ##---------------------------------##
    ## Helper functions
    ##---------------------------------##
    
    def rebuild_ca_metadata(self, modify, task):
        """Wrapper around refresh_pki_metadata"""
        
        if modify:
            if task == 'append':
                ## Get list of all defined CA's
                known_cas = list(CertificateAuthority.objects.all())
                known_cas.append(self)
            elif task == 'replace':
                known_cas = list(CertificateAuthority.objects.exclude(pk=self.pk))
                known_cas.append(self)
            elif task == 'exclude':
                known_cas = list(CertificateAuthority.objects.exclude(pk=self.pk))
        else:
            known_cas = list(CertificateAuthority.objects.all())
        
        ## Rebuild the CA store metadata
        refresh_pki_metadata(known_cas)
    
    ##---------------------------------##
    ## View functions
    ##---------------------------------##
    
    def Tree_link(self):
        
        if PKI_ENABLE_GRAPHVIZ:
            return '<center><a href="%s/pki/tree/%d" target="_blank"><img src="%s/pki/img/tree.png" height="13px" width="13px" alt="Tree" title="Show full CA tree"/></a></center>' % (PKI_BASE_URL, self.pk, os.path.join(PKI_BASE_URL, MEDIA_URL))
        else:
            return '<center><img src="%s/pki/img/tree_disabled.png" alt="Tree" title="Enable setting PKI_ENABLE_GRAPHVIZ"/></center>' % os.path.join(PKI_BASE_URL, MEDIA_URL)
    
    Tree_link.allow_tags = True
    Tree_link.short_description = 'Tree'
  
##------------------------------------------------------------------##
## Certificate class
##------------------------------------------------------------------##

class Certificate(CertificateBase):
    """Certificate model"""
    
    common_name       = models.CharField(max_length=64)
    name              = models.CharField(max_length=64, help_text="Only change the suggestion if you really know what you're doing")
    parent            = models.ForeignKey('CertificateAuthority', blank=True, null=True, help_text='Leave blank to only generate a KEY and CSR')
    passphrase        = models.CharField(max_length=255, null=True, blank=True)
    pf_encrypted      = models.NullBooleanField()
    parent_passphrase = models.CharField(max_length=255, blank=True, null=True)
    pkcs12_encoded    = models.BooleanField(default=False, verbose_name="Create #PKCS12 encoded certificate (additional)")
    pkcs12_passphrase = models.CharField(max_length=255, verbose_name="#PKCS12 passphrase", blank=True, null=True)
    cert_extension    = models.CharField(max_length=64, choices=EXTENSIONS, verbose_name="Purpose")
    subjaltname       = models.CharField(max_length=255, blank=True, null=True, verbose_name="SubjectAltName", \
                                         help_text='Comma seperated list of alt names. Valid are DNS:www.xyz.com, IP:1.2.3.4 and email:a@b.com in any \
                                         combination. Refer to the official openssl documentation for details' )

    class Meta:
        verbose_name_plural = 'Certificates'
        permissions         = ( ( "can_download", "Can download certificate", ), )
        unique_together     = ( ( "name", "parent" ), ("common_name", "parent"), )
    
    def __unicode__(self):
        return self.name
    
    ##---------------------------------##
    ## Redefined functions
    ##---------------------------------##
    
    def save(self):
        """Save the Certificate object"""
        
        if self.pk:
            if self.action in ('update', 'revoke', 'renew'):
                
                action = OpensslActions(self)
                prev   = Certificate.objects.get(pk=self.pk)
                
                if self.action == 'update':
                    
                    ## Create or remove DER certificate
                    if self.der_encoded:
                        action.generate_der_encoded()
                    else:
                        action.remove_der_encoded()
                    
                    ## Create or remove PKCS12 certificate
                    if self.pkcs12_encoded:
                        if prev.pkcs12_encoded and prev.pkcs12_passphrase == self.pkcs12_passphrase:
                            logger.debug( 'PKCS12 passphrase is unchanged. Nothing to do' )
                        else:
                            action.generate_pkcs12_encoded()
                    else:
                        action.remove_pkcs12_encoded()
                        self.pkcs12_passphrase = None
                        prev.pkcs12_passphrase = None
                    
                    if self.pkcs12_passphrase:
                        prev.pkcs12_passphrase = md5_constructor(self.pkcs12_passphrase).hexdigest()
                    else:
                        prev.pkcs12_passphrase = None
                    
                    prev.description    = self.description
                    prev.der_encoded    = self.der_encoded
                    prev.pkcs12_encoded = self.pkcs12_encoded
                    prev.pem_encoded    = True
                    
                elif self.action == 'revoke':
                    
                    ## Revoke and generate CRL
                    action.revoke_certificate(self.parent_passphrase)
                    action.generate_crl(self.parent.name, self.parent_passphrase)
                    
                    ## Modify fields
                    prev.parent_passphrase = None
                    prev.active            = False
                    prev.der_encoded       = False
                    prev.pem_encoded       = False
                    prev.pkcs12_encoded    = False
                    prev.revoked = datetime.datetime.now()
                    
                elif self.action == 'renew':
                    
                    ## Revoke if certificate is active
                    if not action.get_revoke_status_from_cert():
                        action.revoke_certificate(self.parent_passphrase)
                    
                    ## Renew and update CRL
                    action.renew_certificate()
                    action.generate_crl(self.parent.name, self.parent_passphrase)
                    
                    ## Modify fields
                    prev.created = datetime.datetime.now()
                    delta = datetime.timedelta(self.valid_days)
                    prev.expiry_date = datetime.datetime.now() + delta
                    
                    prev.parent_passphrase = None
                    prev.active            = True
                    prev.pem_encoded       = True
                    prev.der_encoded       = self.der_encoded
                    prev.pkcs12_encoded    = self.pkcs12_encoded
                    prev.revoked           = None
                    prev.valid_days = self.valid_days
                    
                    ## Get the new serial
                    prev.serial     = action.get_serial_from_cert()
                    #prev.passphrase = md5_constructor(self.passphrase).hexdigest()
                
                ## Save the data
                self = prev
                self.action = 'update'
                
                super(Certificate, self).save()
        else:
            ## Set creation data
            self.created = datetime.datetime.now()
            delta = datetime.timedelta(self.valid_days)
            self.expiry_date = datetime.datetime.now() + delta
            
            ## Force instance to be active
            self.active = True
            
            logger.info( "***** { New certificate generation: %s } *****" % self.name )
            
            ## Generate key and certificate
            action = OpensslActions(self)
            
            action.generate_key()
            action.generate_csr()
            action.sign_csr()
            
            ## Get the serial from certificate
            self.serial = action.get_serial_from_cert()
            
            self.ca_chain = self.parent.ca_chain
            if self.ca_chain == 'self-signed':
                self.ca_chain = self.parent.name
            
            self.pem_encoded = True
            
            ## Create or remove DER certificate
            if self.der_encoded:
                action.generate_der_encoded()
            else:
                action.remove_der_encoded()
            
            ## Create or remove PKCS12 certificate
            if self.pkcs12_encoded:
                action.generate_pkcs12_encoded()
            else:
                action.remove_pkcs12_encoded()
            
            if self.pkcs12_passphrase:
                self.pkcs12_passphrase = md5_constructor(self.pkcs12_passphrase).hexdigest()
            
            ## Encrypt passphrase and blank parent's passphrase
            if self.passphrase:
                self.passphrase = md5_constructor(self.passphrase).hexdigest()
            
            self.parent_passphrase = None
            
            ## Save the data
            super(Certificate, self).save()
    
    def delete(self, passphrase):
        """Delete the Certificate object"""
        
        ## Remoke first ca in the chain
        c_action = OpensslActions(Certificate.objects.get(pk=self.pk))
        c_action.revoke_certificate(passphrase)
        c_action.generate_crl(ca=self.parent.name, pf=passphrase)
        
        ## Time for some rm action
        a = OpensslActions(self)
        a.remove_complete_certificate()
        
        ## Call the "real" delete function
        super(Certificate, self).delete()

    



## django-pki exception middleware

from django import http
from django.conf import settings
from logging import getLogger
from django.core.urlresolvers import RegexURLResolver 

from pki.settings import PKI_BASE_URL

import sys

logger = getLogger("pki")

def resolver(request):
    """
    Returns a RegexURLResolver for the request's urlconf.

    If the request does not have a urlconf object, then the default of
    settings.ROOT_URLCONF is used.
    """
    from django.conf import settings
    urlconf = getattr(request, "urlconf", settings.ROOT_URLCONF)
    return RegexURLResolver(r'^/', urlconf)

class PkiExceptionMiddleware(object):
    """Exception logging moddleware for django-pki.
       Based on http://djangosnippets.org/snippets/638/"""
    
    def process_exception(self, request, exception):
        if isinstance(exception, http.Http404):
            return self.handle_404(request, exception)
        else:
            return self.handle_500(request, exception)
    
    def handle_404(self, request, exception):
        if settings.DEBUG:
            from django.views import debug
            return debug.technical_404_response(request, exception)
        else:
            callback, param_dict = resolver(request).resolve404()
            return callback(request, **param_dict)
    
    def handle_500(self, request, exception):        
        exc_info = sys.exc_info()
        if settings.DEBUG:
            return self.debug_500_response(request, exception, exc_info)
        else:
            self.log_exception(request, exception, exc_info)
            return self.production_500_response(request, exception, exc_info)
    
    def debug_500_response(self, request, exception, exc_info):
        from django.views import debug
        return debug.technical_500_response(request, *exc_info)
    
    def production_500_response(self, request, exception, exc_info):
        '''Return an HttpResponse that displays a friendly error message.'''
        callback, param_dict = resolver(request).resolve500()
        return callback(request, **param_dict)
    
    def log_exception(self, request, exception, exc_info):
        
        logger.error( '' )
        logger.error( '=============================================================' )
        logger.error( ' Exception ' )
        logger.error( '=============================================================' )
        logger.error( '' )
        logger.error( 'REMOTE_ADDR: %s' % request.META.get('REMOTE_ADDR') )
        logger.error( 'REQUEST_URI: %s' % request.META.get('REQUEST_URI') )
        logger.error( 'HTTP_REFERER: %s' % request.META.get('HTTP_REFERER') )
        logger.error( 'HTTP_USER_AGENT: %s' % request.META.get('HTTP_USER_AGENT') )
        logger.error( '' )
        logger.error( _get_traceback(exc_info) )
        #logger.error( 'REQUEST_URI: %s' % repr(request) )

def _get_traceback(self, exc_info=None):
    """Helper function to return the traceback as a string"""
    import traceback
    return '\n'.join(traceback.format_exception(*(exc_info or sys.exc_info())))



import os
import sys

from distutils.core import setup
from distutils.command.install import INSTALL_SCHEMES
from distutils.command.install_data import install_data

from pki import __version__ as version

root_dir = os.path.abspath(os.path.dirname(__file__))

def get_file_contents(f):
    try:
        return open(os.path.join(root_dir, f), 'rb').read().decode('utf-8').strip()
    except IOError:
        return 'UNKNOWN'

def list_data_files(d):
    data_files = []
    for root, dirs, files in os.walk(os.path.join(root_dir, d)):
        path = root.replace(root_dir + os.sep, '', 1)
        if len(files) > 0:
            data_files.append((path, [os.path.join(path, f) for f in files]))

    return data_files

# Tell distutils to put data_files in platform-specific installation
for scheme in INSTALL_SCHEMES.values(): scheme['data'] = scheme['purelib']

# Fix MacOS platform-specific lib dir
class osx_install_data(install_data):
    def finalize_options(self):
        self.set_undefined_options('install', ('install_lib', 'install_dir'))
        install_data.finalize_options(self)

if sys.platform == "darwin":
    cmdclasses = {'install_data': osx_install_data}
else:
    cmdclasses = {'install_data': install_data}

setup(
    name = 'django-pki',
    version=version,
    description = 'A PKI based on the Django admin',
    long_description=get_file_contents('README.markdown'),
    author='Daniel Kerwin',
    author_email='daniel@linuxaddicted.de',
    url='http://www.github.com/dkerwin/django-pki/',
    license='GPL',
    download_url='http://pypi.python.org/packages/source/d/django-pki/django-pki-%s.tar.gz' % version,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Telecommunications Industry',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Security',
        'Topic :: Security :: Cryptography',
        'Topic :: System :: Systems Administration',
    ],
    packages = ['pki', 'pki.templatetags'],
    data_files = list_data_files('pki/media') + list_data_files('pki/templates'),
    cmdclass = cmdclasses,
)
